/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "Galaxy.h"
#include "GalaxyLoader.h"

CGalaxy::CGalaxy(void)
{
}

CGalaxy::~CGalaxy(void)
{
}

int CGalaxy::AddBase(CBase *in_Base)
{
	int index_i;
	index_i = AddMent((CEntity*) in_Base);
	bases.AddToEnd(index_i);
	return bases.Size() -1;
}

int CGalaxy::AddShip(CShip *in_Ship)
{
	int index_i;
	index_i = AddMent((CEntity*) in_Ship);
	ships.AddToEnd(index_i);
	return ships.Size() -1;
}

int CGalaxy::AddInert(CInert *in_Inert)
{
	int index_i;
	index_i = AddMent((CEntity*) in_Inert);
	inerts.AddToEnd(index_i);
	return inerts.Size() -1;
}

int CGalaxy::LoadFromFile(string in_Filename_s)
{
	CGalaxyLoader gl;
	return gl.Load(in_Filename_s, this);
}

int CGalaxy::Render()
{
	glColor4f(1.0,1.0,1.0,1.0);
        glClearColor(0,0,0,0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	// do thi better some day...
	// set up drawing and render each entity
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	// gluPerspective(45, 640/480, 1, 100);
	glFrustum(-1, 1, -0.75, 0.75, 1, 1000);
    glMatrixMode(GL_MODELVIEW);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    // enable depth testing if it's not already enabled.
    glEnable(GL_DEPTH_TEST);
	// set up camera
    CVec4f u, f, p;
    u = camera.GetUp();

    f = camera.GetFace();
    p = camera.GetPos();
	gluLookAt(p.X(),p.Y(),p.Z(),
			  p.X() + f.X(), p.Y() + f.Y(),p.Z()+f.Z(),
			  u.X(),u.Y(),u.Z());

	// draw an outline of the arena
	glLineWidth(2);
	glDisable(GL_LIGHTING);
	glDisable(GL_BLEND);
	glBegin(GL_LINE_LOOP);
		glColor4f(1.0,0.0,0.0,1.0);
		glVertex3f(0,0,0);
		glVertex3f(width_f, 0, 0);
		glVertex3f(width_f, height_f, 0);
		glVertex3f(0, height_f, 0);
	glEnd();

	// draw the background
	extern CTextureManager texMan;

	glEnable(GL_TEXTURE_2D);
	glDepthMask(GL_FALSE);
	glColor4f(1.0,1.0,1.0,1.0);
	glBindTexture(GL_TEXTURE_2D, texMan.textures[this->backTex_i].id_ui);
	CVec4f campos = camera.GetPos();
	glBegin(GL_QUADS);
		glTexCoord2f(1, 1); glVertex3f(-600,-600, campos.Z()-500);
		glTexCoord2f(0, 1); glVertex3f(width_f+600,-600, campos.Z()-500);
		glTexCoord2f(0, 0); glVertex3f(width_f+600,height_f+600, campos.Z()-500);
		glTexCoord2f(1, 0); glVertex3f(-600,height_f+600, campos.Z()-500);
	glEnd();
	glFlush();
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE);
	glColor4f(1.0,1.0,1.0,0.5);
	glBindTexture(GL_TEXTURE_2D, texMan.textures[this->starsTex_i].id_ui);
	glBegin(GL_QUADS);
		glTexCoord2f(1, 1); glVertex3f(campos.X()-75,campos.Y()-75, campos.Z()-100);
		glTexCoord2f(0, 1); glVertex3f(campos.X()+75,campos.Y()-75, campos.Z()-100);
		glTexCoord2f(0, 0); glVertex3f(campos.X()+75,campos.Y()+75, campos.Z()-100);
		glTexCoord2f(1, 0); glVertex3f(campos.X()-75,campos.Y()+75, campos.Z()-100);
	glEnd();
	glFlush();
	glDisable(GL_BLEND);
	glDepthMask(GL_TRUE);
	glDisable(GL_TEXTURE_2D);


	glColor4f(1.0, 1.0, 1.0, 1.0);
	glEnable(GL_LIGHTING);
	
	CScene::Render();


	galPartGen.Render();
	glFinish();
	// draw a map of the arena?
	// or maybe something more like a radar??
	return 0;
}

void CGalaxy::Update()
{
	galPartGen.Update();
}

void CGalaxy::FreeAll()
{
	ents.Clear();
	ships.Clear();
	shipsAvailToPlayer.Clear();
	ments.Clear();
	lights.Clear();
	bases.Clear();
	inerts.Clear();
	shipTypes.Clear();
	gunTypes.Clear();
	cargoTypes.Clear();
	factions.Clear();
}
