/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "GalaxyLoader.h"
#include "ShipLoader.h"
#include <algorithm>
using namespace std;

#define MAX_INERT_CONTENT_QUANTITY 200

CGalaxyLoader::CGalaxyLoader(void)
{
}

CGalaxyLoader::~CGalaxyLoader(void)
{
}

CVec4f CGalaxyLoader::ReadPos()
{
	CVec4f out_v4f;
	float one_f, two_f;
	inFile >> one_f >> two_f;
	out_v4f.SetXYZW(one_f, two_f, 0, 1);
	return out_v4f;
}

void CGalaxyLoader::NextLine()
{
	char ch = ' ';
	while((ch != '\n') && (!inFile.eof()))
	{
		inFile.get(ch);
	}
}

int CGalaxyLoader::ReadType()
{
	
	string s;
	s = ReadToken<string> ();
/* Soo, gcc is not infallable - here's a wonderful toy that seems to be fuck-arsed... */
//	transform(s.begin(), s.end(), s.begin(), toupper);
	string::iterator si;
	si = s.begin();
	while(si != s.end())
	{
		*si = toupper(*si);
		si++;
	}

	if(s == "#")  // comment
		return -1;
	else if(s == "CARGOT")
		return CARGOT;
	else if(s == "GUNT")
		return GUNT;
	else if(s == "SHIPT")
		return SHIPT;
	else if(s == "WIDTH")
		return WIDTH;
	else if(s == "HEIGHT")
		return HEIGHT;
	else if(s == "BASE")
		return BASE;
	else if(s == "RBASE")
		return RBASE;
	else if(s == "RSHIP")
		return RSHIP;
	else if(s == "RINERT")
		return RINERT;
	else if(s == "FACT")
		return FACTION;
	else if(s == "NUMFACT")
		return NUMFACT;
	else if(s == "SHIP")
		return SHIP;
	else return -2;			// unknown/syntax error.
	
}

int CGalaxyLoader::Load(string in_Filename_s, CGalaxy *in_Galaxy)
{
	inFile.clear();
	inFile.open(in_Filename_s.c_str());
	if(!inFile.is_open())
	{
		cout << "Error loading file: " << in_Filename_s << endl;
		exit(1);
	}
	inFile.seekg(0, ios::beg);
	in_Galaxy->filename_s = in_Filename_s;
	int type_i;
	int shipTypes_i= 0;
	while(!inFile.eof())
	{
		type_i = ReadType();
		switch(type_i)
		{
		case -1: {NextLine(); break;}
		case -2: {NextLine(); break;}
		case CARGOT:
			{
				SCargoType *ct;
				ct = new(SCargoType);
				string str;
				ct->name = ReadToken<string>();
				str = ReadToken<string>();
				if(str == "AGRIC")
					ct->type_i = AGRIC;
				else if(str == "RAWMAT")
					ct->type_i = RAWMAT;
				else if(str == "LUXURY")
					ct->type_i = LUXURY;
				else if(str == "FUEL")
					ct->type_i = FUEL;
				else if(str == "REFINED")
					ct->type_i = REFINED;
				else if(str == "TECH")
					ct->type_i = TECH;
				ct->standardValue_f = ReadToken<float>();
				ct->maxDeviation_f  = ReadToken<float>();
				ct->volumePerTonne_i= ReadToken<int>();
				in_Galaxy->cargoTypes.AddToEnd(ct);
				NextLine();
				break;
			}
		case GUNT:
			{
				SGunType *gt;
				gt = new(SGunType);
				gt->name = ReadToken<string>();
				string str;
				str = ReadToken<string>();
				if(str != "NULL")
				{
					extern CTextureManager texMan;
					gt->tex_i = texMan.LoadTexture(str, WIN_BMP);
					gt->hasTexturedParticle_b = true;
				}
				else
				{
					gt->tex_i = -1;
					gt->hasTexturedParticle_b = false;
				}
				gt->r_f = ReadToken<float>();
				gt->g_f = ReadToken<float>();
				gt->b_f = ReadToken<float>();
				gt->a_f = ReadToken<float>();
				gt->damage_i = ReadToken<int>();
				gt->speed_f  = ReadToken<float>();
				gt->chargeRate_f = ReadToken<float>();
				gt->chargeCost_f = ReadToken<float>();
				gt->maxCharge_f  = ReadToken<float>();
				gt->minCharge_f  = ReadToken<float>();
				gt->fireRate_i   = ReadToken<int>();
				gt->absorbtion_i = ReadToken<int>();
				string str2;
				str2 = ReadToken<string>();
				if(str2 == "BURST")
				{
					gt->isBurst_b = true;
				}
				else gt->isBurst_b = false;
				gt->partSize_f = ReadToken<float>();
				in_Galaxy->gunTypes.AddToEnd(gt);
				NextLine();
				break;
			}
		case SHIPT:
			{
				shipTypes_i++;
				CShipLoader sl;
				string str;
				str = ReadToken<string>();
				sl.Load(str, in_Galaxy);
				str = ReadToken<string>();
				if(str == "P")
					in_Galaxy->shipsAvailToPlayer.AddToEnd(shipTypes_i-1);
				NextLine();
				break;
			}
		case WIDTH:
			{
				in_Galaxy->width_f = ReadToken<float>();
				NextLine();
				break;
			}
		case HEIGHT:
			{
				in_Galaxy->height_f = ReadToken<float>();
				NextLine();
				break;
			}
		case BASE:
			{
				CBase *b;
				b = new(CBase);
				b->name = ReadToken<string>();
				CVec4f pos;
				pos = ReadPos();
				b->SetPos(pos.X(), pos.Y(), 0);
				b->agriculture_i  = ReadToken<int>();
				b->rawMaterials_i = ReadToken<int>();
				b->luxuryItems_i  = ReadToken<int>();
				b->fuel_i         = ReadToken<int>();
				b->refinedGoods_i = ReadToken<int>();
				b->techGoods_i    = ReadToken<int>();
				b->techLevel_i    = ReadToken<int>();
				b->LoadMesh("base.obj", OBJ);
				b->LoadTexture("base.bmp", WIN_BMP);
				in_Galaxy->AddBase(b);
				NextLine();
				break;
			}
		case RBASE:
			{
				int basesToGenerate_i;
				basesToGenerate_i = ReadToken<int>();
				NextLine();
				for(int bc = 0; bc < basesToGenerate_i; bc++)
				{
					CBase *nb;
					nb = new(CBase);
					nb->name = "Station" + Cast<string> (rand()%200) + "-" + Cast<string> (rand()%200);
					nb->agriculture_i  = rand()%10;
					nb->rawMaterials_i = rand()%10;
					nb->luxuryItems_i  = rand()%10;
					nb->fuel_i         = rand()%10;
					nb->refinedGoods_i = rand()%10;
					nb->techGoods_i    = rand()%10;
					nb->techLevel_i    = rand()%3;
					nb->SetPos(rand()%(int)in_Galaxy->width_f, rand()%(int)in_Galaxy->height_f, 0);
					nb->SetVelocity(0,0,0);
					nb->SetRotation(0,0,0);
					in_Galaxy->AddBase(nb);
				}
				NextLine();
				break;
			}
		case RSHIP:
			{
				int shipsToGenerate_i;
				shipsToGenerate_i = ReadToken<int>();
				NextLine();
				for(int sc = 0; sc < shipsToGenerate_i; sc++)
				{
					CShip *ns;
					ns = new(CShip);
					ns->shipType_i = rand()%in_Galaxy->shipTypes.Size();
					ns->mesh_p = in_Galaxy->shipTypes[ns->shipType_i]->mesh_p;
					ns->CalculateBoundSph();
					ns->SetPos(rand()%(int)in_Galaxy->width_f, rand()%(int)in_Galaxy->height_f, 0);
					
					for(int gc_i = 0; gc_i < in_Galaxy->shipTypes[ns->shipType_i]->gunPositions.Size(); gc_i++)
					{
						SGun tempGun;
						tempGun.currentCharge_f = 100;
						tempGun.gunType_i = rand()%in_Galaxy->gunTypes.Size();
						tempGun.isActive_b = true;
						tempGun.reloadTime_i = 0;
						ns->guns.AddToEnd(tempGun);
					}

					ns->faction_i = rand()%(in_Galaxy->factions.Size()-1)+1;
					// This is going to be much more complicated...?

					ns->mk1Shield_i= rand()%2;
					in_Galaxy->AddShip(ns);
				}
				NextLine();
				break;
			}
		case RINERT:
			{
				int inertToGenerate_i;
				inertToGenerate_i = ReadToken<int>();
				NextLine();
				for(int ic = 0; ic < inertToGenerate_i; ic++)
				{
					CInert *ni;
					ni = new(CInert);
					ni->LoadMesh("data/rock.obj", OBJ);
					ni->LoadTexture("data/rock.bmp", WIN_BMP);
					//ni->cargoType_i = rand()%in_Galaxy->cargoTypes.Size();
					ni->quantity_i  = rand()%MAX_INERT_CONTENT_QUANTITY;
					ni->SetPos(rand()%(int)in_Galaxy->width_f, rand()%(int)in_Galaxy->height_f, 0);
					ni->SetRotation(rand()%360, rand()%360, rand()%360);
					ni->SetVelocity(rand()%3/10, rand()%3/10, 0);
					ni->CalculateBoundSph();
					in_Galaxy->AddInert(ni);
				}
				break;
			}
		case NUMFACT:
			{
				// set it so that there is independant + x factions
				// where x is the number read in.
				in_Galaxy->factions.SetSize(ReadToken<int>() + 1);
				for(int fc_i = 0; fc_i < in_Galaxy->factions.Size(); fc_i++)
				{
					in_Galaxy->factions[fc_i] = NULL;
				}
				// create independant faction
				in_Galaxy->factions[0] = new(SFaction);
				in_Galaxy->factions[0]->name = "Independant";
				for(int rc_i =0; rc_i < in_Galaxy->factions.Size(); rc_i++)
				{
					in_Galaxy->factions[0]->relations.AddToEnd(1);
				}
				in_Galaxy->factions[0]->relations[0] = 10;

				break;
			}
		case FACTION:
			{
				// find first NULL entry in array
				int c_i = 0;
				int numShips_i;
				while(in_Galaxy->factions[c_i] != NULL)
				{
					c_i++;
				}
				in_Galaxy->factions[c_i] = new(SFaction);
				in_Galaxy->factions[c_i]->name = ReadToken<string>();
				numShips_i = ReadToken<int>();
				for(int rc_i =0; rc_i < in_Galaxy->factions.Size(); rc_i++)
				{
					in_Galaxy->factions[c_i]->relations.AddToEnd(ReadToken<int>());
				}

				// generate faction ships
				for(int sc = 0; sc < numShips_i; sc++)
				{
					CShip *ns;
					ns = new(CShip);
					ns->shipType_i = rand()%in_Galaxy->shipTypes.Size();
					ns->mesh_p = in_Galaxy->shipTypes[ns->shipType_i]->mesh_p;
					ns->CalculateBoundSph();
					ns->SetPos(rand()%(int)in_Galaxy->width_f, rand()%(int)in_Galaxy->height_f, 0);
					
					for(int gc_i = 0; gc_i < in_Galaxy->shipTypes[ns->shipType_i]->gunPositions.Size(); gc_i++)
					{
						SGun tempGun;
						tempGun.currentCharge_f = 100;
						tempGun.gunType_i = rand()%in_Galaxy->gunTypes.Size();
						tempGun.isActive_b = true;
						tempGun.reloadTime_i = 0;
						ns->guns.AddToEnd(tempGun);
					}

					ns->faction_i = c_i;
					// This is going to be much more complicated...?

					ns->mk1Shield_i= rand()%2;
					in_Galaxy->AddShip(ns);
				}
				break;
			}
		case SHIP:
			{
				CShip *ns;
				ns = new(CShip);
				ns->faction_i = ReadToken<int>();
				ns->shipType_i = ReadToken<int>();

				ns->mesh_p = in_Galaxy->shipTypes[ns->shipType_i]->mesh_p;
				ns->CalculateBoundSph();
				ns->SetPos(rand()%(int)in_Galaxy->width_f, rand()%(int)in_Galaxy->height_f, 0);
				
				ns->mk1Shield_i = ReadToken<int>();
				ns->mk2Shield_i = ReadToken<int>();
				ns->mk3Shield_i = ReadToken<int>();

				for(int gc_i = 0; gc_i < in_Galaxy->shipTypes[ns->shipType_i]->gunPositions.Size(); gc_i++)
				{
					SGun tempGun;
					tempGun.currentCharge_f = 100;
					tempGun.gunType_i = ReadToken<int>();
					tempGun.isActive_b = true;
					tempGun.reloadTime_i = 0;
					ns->guns.AddToEnd(tempGun);
				}

				in_Galaxy->AddShip(ns);
				NextLine();
				break;
			}
		}
	}
	return 0;
}
