/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "GunParticle.h"
#include "FreeBooter.h"
#include "Galaxy.h"
#include "Dazzle.h"
#include <iostream>
using namespace std;

CGunParticle::CGunParticle(void)
{
	this->size_f = 0.75;
	this->a_f = 1.0;
	this->b_f = 1.0;
	this->g_f = 1.0;
	this->r_f = 1.0;
}

CGunParticle::~CGunParticle(void)
{
	if(inContact_b)
	{
		contactThis->inContact_b = false;
		contactThis->contactThis = NULL;
	}
}

void CGunParticle::Update()
{
	CSimpleParticle::Update();

	// check for collision with other entities
	extern CFreebooter FB;
	CGalaxy *gal;
	gal = (CGalaxy*) FB.scene;

	for(int sc_i = 0; sc_i < gal->ships.Size(); sc_i++)
	{
		if(this->owner != gal->ents[gal->ships[sc_i]])
		{
			if(this->BSBSCollision((CModelEntity*)gal->ents[gal->ships[sc_i]]) == true)
			{
				float pain_f, abs_f;
				pain_f = gal->gunTypes[this->gunType_i]->damage_i;
				abs_f = (float) gal->gunTypes[this->gunType_i]->absorbtion_i / 100 ;
				

				
				extern CTextureManager texMan;
				int texInd = texMan.LoadTexture("data/shot.bmp",WIN_BMP);

				this->life = 0;
				CShip *shp = (CShip*)gal->ents[gal->ships[sc_i]];
				float sdam = abs_f * pain_f;
				int mxSldStr_i = (25*shp->mk1Shield_i) + (45*shp->mk2Shield_i) + (65*shp->mk3Shield_i);
				if((shp->shieldDamage_f += sdam) > mxSldStr_i)
				{
					shp->damage_i += shp->shieldDamage_f - mxSldStr_i;
					shp->shieldDamage_f = mxSldStr_i;
					// hit primarily off the hull, give a particle flare
					for(int npc_i = 0; npc_i < rand()%25+5; npc_i++)
					{
						CSimpleParticle *newPart= NULL;
						try
						{
							newPart = new(CSimpleParticle);
							if(newPart != NULL)
							{
								newPart->SetPos(X(), Y(), 0);
								newPart->SetVelocity(((float)(rand()%1000))/2000 - 0.25,
													((float)(rand()%1000))/2000 - 0.25,
													((float)(rand()%1000))/2000 - 0.25);
								newPart->r_f = 1;
								newPart->b_f = 0.2;
								newPart->g_f = 0.8;
								newPart->a_f = 0.8;
								newPart->life = 70;

								newPart->tex_i = texInd;
								newPart->hasTexture_b = true;
								newPart->size_f = rand()%3+1;
		                        
								gal->galPartGen.AddParticle(newPart);
							}
						}
						catch(bad_alloc)
						{
							cout << "Couldn't allocate particle - damage flare" << endl;
						}
					}
				}
				else if((shp->shieldDamage_f += sdam) < mxSldStr_i)
				{
					// hit primarily off the shields, give a particle flare
					for(int npc_i = 0; npc_i < rand()%25+5; npc_i++)
					{
						CSimpleParticle *newPart = NULL;
						try
						{
							newPart = new(CSimpleParticle);
							if(newPart != NULL)
							{
								newPart->SetPos(X(), Y(), 0);
								newPart->SetVelocity(((float)(rand()%1000))/4000 - 0.012,
													((float)(rand()%1000))/4000 - 0.012,
													((float)(rand()%1000))/2000 - 0.25);
								newPart->r_f = 0.2;
								newPart->b_f = 1;
								newPart->g_f = 0.4;
								newPart->a_f = 0.8;
								newPart->life = 70;
								newPart->size_f = (rand()%40)/20+1;
								
								newPart->tex_i = texInd;
								newPart->hasTexture_b = true;

								gal->galPartGen.AddParticle(newPart);
							}
						}
						catch(bad_alloc)
						{
							cout << "couldn't allocate particle - shield flare" << endl;
						}
					}
				}
				float hdam = (1-abs_f) * pain_f;
				shp->damage_i += hdam;
				if(shp->damage_i > gal->shipTypes[shp->shipType_i]->maxHullHealth_i)
				{
					if(shp->dead_b == false)
					{
						this->owner->kills_i++;
						shp->dead_b = true;
						shp->aiStartTime_i = SDL_GetTicks();
						shp->aiSubMode_i   = 1000;
					}
				}
			}
		}
	}
	for(int sc_i = 0; sc_i < gal->inerts.Size(); sc_i++)
	{
		if(this->BSBSCollision((CModelEntity*)gal->ents[gal->inerts[sc_i]]) == true)
		{
			this->life = 0;
		}
	}

}

void CGunParticle::Render()
{
    if(this->life > 0)
	{
		glPushMatrix();
		glDisable(GL_CULL_FACE);
		glDisable(GL_LIGHTING);
		extern CFreebooter FB;
		CGalaxy *gal = (CGalaxy*)FB.scene;
		if(gal->gunTypes[this->gunType_i]->hasTexturedParticle_b)
		{
			glEnable(GL_TEXTURE_2D);
			extern CTextureManager texMan;
			glBindTexture(GL_TEXTURE_2D, texMan.textures[gal->gunTypes[this->gunType_i]->tex_i].id_ui);
		}
		else glDisable(GL_TEXTURE_2D);

		glTranslatef(pos_v4f.X(), pos_v4f.Y(), pos_v4f.Z());
		//glRotatef(rot_v3f.X(), 1, 0, 0);
		//glRotatef(rot_v3f.Y(), 0, 1, 0);
		//glRotatef(rot_v3f.Z(), 0, 0, 1);

		glEnable(GL_BLEND);
		//glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		glDepthMask(GL_FALSE);
		glColor4f((GLfloat)r_f,(GLfloat) g_f,(GLfloat) b_f,(GLfloat) a_f);
		glBegin(GL_QUADS);
			glNormal3f(0,0,1);
			glTexCoord2i(0,0); glVertex3f(-size_f, -size_f, 0);
			glTexCoord2i(1,0); glVertex3f( size_f,-size_f, 0);
			glTexCoord2i(1,1); glVertex3f( size_f,size_f, 0);
			glTexCoord2i(0,1); glVertex3f(-size_f,size_f, 0);
			/*glNormal3f(0,1,0);
			glTexCoord2i(0,0); glVertex3f(-size_f,0, -size_f);
			glTexCoord2i(1,0); glVertex3f( size_f,0, -size_f);
			glTexCoord2i(1,1); glVertex3f( size_f,0,  size_f);
			glTexCoord2i(0,1); glVertex3f(-size_f,0,  size_f);*/
		glEnd();
		/*float cols[4];
		glGetFloatv(GL_CURRENT_COLOR, cols);
		fprintf(stdout, "%f %f %f %f \n", cols[0], cols[1], cols[2], cols[3]);*/

		glDisable(GL_TEXTURE_2D);
		glColor4f(1.0,1.0,1.0,1.0);
		glDepthMask(GL_TRUE);
		glDisable(GL_BLEND);
		glEnable(GL_LIGHTING);
		glEnable(GL_CULL_FACE);
		glPopMatrix();
	}
}

