/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "FreeBooter.h"
#include "SDL.h"
#include "FTGLTextureFont.h"
#include "EquipDefines.h"
#include "savegame.h"
#include "Dazzle.h"
#include <algorithm>


#define MAIN_MENU 0
#define GAME_MENU 1
#define KIT_OUT_SCREEN 2
#define IN_LEVEL 3
#define LOADING 4
#define DEMO 5
#define SCORES 6
#define NUM_DEM_LEV 3

#define LAST_LEVEL  20
#define FIRST_LEVEL 1


extern CFreebooter FB;
CGalaxy* gal;

struct SPlayer
{
	CShip *ship_p;
	int score_i;
	int kills_i;
	int currentLevel_i;
	int lives_i;
};

vector<int> highScores;

SPlayer player;
int currentScreen_i;
CTextBox tb;
CDynArray<float> cargVals;
CDynArray<int>   cargQuants;
CDynArray<int>   playerCargQuants;

bool MainMenu();
void HighScores();
void KitOut();
void LoadLevel(int level);
bool GameMenu();
bool Fly();
bool Demo();
float CalcCarVal(int rating, float stdval, float maxdev);


int RunFreebooter()
{
	ifstream scoresFile;
	scoresFile.open("scores");
	for(int hsc_i = 0; hsc_i < 10; hsc_i++)
	{
		int i;
		scoresFile>>i;
		highScores.push_back(i);
	}
	scoresFile.close();
	sort(highScores.begin(), highScores.end());
	

	gal = (CGalaxy*)FB.scene;
	// so here's the start
	currentScreen_i = MAIN_MENU;

	player.currentLevel_i = 2;

	glClearColor(0,0,0,0);
	bool levelWin_b = false;
	bool quit_b = false;
	while(quit_b == false)
	{
		if(currentScreen_i == MAIN_MENU) quit_b = MainMenu();
		else if(currentScreen_i == GAME_MENU) quit_b = GameMenu();
		else if(currentScreen_i == KIT_OUT_SCREEN) KitOut();
		else if(currentScreen_i == LOADING) LoadLevel(player.currentLevel_i+1);
		else if(currentScreen_i == SCORES) HighScores();
		else if(currentScreen_i == DEMO)
		{
			if(Demo() == false) currentScreen_i = MAIN_MENU;
			else currentScreen_i = DEMO;
		}
		else if(currentScreen_i == IN_LEVEL)
		{
			int timeForLevel;
			timeForLevel = SDL_GetTicks();
			levelWin_b = Fly();
			timeForLevel = SDL_GetTicks() - timeForLevel;
			player.kills_i += player.ship_p->kills_i;
			if((levelWin_b == false) && (currentScreen_i == IN_LEVEL))
			{
				player.currentLevel_i--; //try again
				player.lives_i--;
				currentScreen_i = LOADING;
			}
			if(levelWin_b == true)
			{
				player.score_i += (int)((player.ship_p->kills_i * 50000)/timeForLevel) + 
				                        (player.ship_p->kills_i) + 
									    (player.currentLevel_i * 100) - 
										(player.ship_p->damage_i * 3);
                currentScreen_i = LOADING;
				// new life after three levels
				if((player.currentLevel_i % 4) == 0)player.lives_i++;
			    if(player.currentLevel_i == LAST_LEVEL) 
				{
					currentScreen_i = MAIN_MENU;	// victory!!
					highScores.push_back(player.score_i);
					sort(highScores.begin(), highScores.end());
					// show high scores
					HighScores();
				}
			}
			if(((levelWin_b == false) && (currentScreen_i == MAIN_MENU)) || (player.lives_i < 0))
			{
				highScores.push_back(player.score_i);
				sort(highScores.begin(), highScores.end());
				// show high scores
				HighScores();
				currentScreen_i = MAIN_MENU;
			}
		}
	}

	// clean things up, save high scores, and end program
	// save high scores
	ofstream scoresFile2;
	scoresFile2.open("scores");
	for(int hsc_i = highScores.size()-10; hsc_i < highScores.size()-0; hsc_i++)
	{
		scoresFile2 << highScores[hsc_i]<< " ";		
	}
	scoresFile.close();

	return 0;
}

bool MainMenu()
{
	int selection = 0;

	
	tb.font->FaceSize(20);
	SDL_Delay(500);
	SDL_PumpEvents();
	Uint8 *keystate;
	keystate = SDL_GetKeyState(NULL);
    while(keystate[SDLK_RETURN] == 0)
	{
		SDL_PumpEvents();
		glClear(GL_COLOR_BUFFER_BIT);
		// draw highlight
		if(selection == 0) tb.RenderBG(495, 498, 0.8, 0, 0, 200);
		else if(selection == 1) tb.RenderBG(495, 448, 0.8, 0, 0, 200);
		else if(selection == 2) tb.RenderBG(495, 398, 0.8, 0, 0, 200);
		else if(selection == 3) tb.RenderBG(495, 348, 0.8, 0, 0, 200);

		// draw menu
		tb.RenderText(500, 500, "NEW GAME");
		tb.RenderText(500, 450, "HIGH SCORES");
		tb.RenderText(500, 400, "DEMO");
		tb.RenderText(500, 350, "QUIT");

		if(keystate[SDLK_UP]==1)
		{
			if(selection > 0)selection--; 
			SDL_GL_SwapBuffers();
			SDL_Delay(100);
		}
		else if(keystate[SDLK_DOWN] == 1) 
		{
			if(selection < 3)selection++;
			SDL_GL_SwapBuffers();
			SDL_Delay(100);
		}
		else SDL_GL_SwapBuffers();

		
        
	}

	if(selection == 0) 
	{ 
		currentScreen_i = LOADING; 
		player.currentLevel_i = FIRST_LEVEL-1;
		player.lives_i = 3;
		player.kills_i = 1;
		player.score_i = 0;
	}
	if(selection == 1)
	{
		currentScreen_i = SCORES;
	}
	if(selection == 2) currentScreen_i = DEMO;
	if(selection == 3) return true;	// we're quitting
	return false;		
	
}

void HighScores()
{
	tb.font->FaceSize(20);
	// show high scores
	SDL_Delay(100);
	SDL_PumpEvents();
	Uint8 *keystate;
	keystate = SDL_GetKeyState(NULL);
	while(keystate[SDLK_RETURN] == 0)
	{
		glClear(GL_COLOR_BUFFER_BIT);
		for(int hsc_i = highScores.size() - 10; hsc_i < highScores.size(); hsc_i++)
		{
			if(hsc_i < highScores.size())
			{
				if(player.score_i == highScores[hsc_i])
					tb.RenderBG(100, 298 + (hsc_i*20), 0.8, 0, 0, 100);
				tb.RenderText(100, 300 + (hsc_i*20), Cast<string>(highScores[hsc_i]));
			}
			else tb.RenderText(100, 300 + (hsc_i*20), Cast<string>(0));

		}
		SDL_PumpEvents();
		SDL_GL_SwapBuffers();
	}
	currentScreen_i = MAIN_MENU;
	
}

void LoadLevel(int level)
{
	tb.font->FaceSize(20);
	// clear out old level data
	delete (player.ship_p);
	gal->FreeAll();

	string str;
	glClear(GL_COLOR_BUFFER_BIT);
	if(level > -1)
	{
        tb.RenderText(10, 100, "Loading level "+Cast<string>(level)+"....");
		SDL_GL_SwapBuffers();
		str = "data/levels/level" +Cast<string>(level)+".dvl";
	}
	if(level == -1)
	{
	    tb.RenderText(10, 100, "Loading demo level....");
		SDL_GL_SwapBuffers();
		str = "data/levels/demo" + Cast<string>(rand()%NUM_DEM_LEV) + ".dvl";
	}
	//+ Cast<string>(level) )+".dlv";
	gal->LoadFromFile(str);

	CLightEntity *light;
	light = new(CLightEntity);
	light->diffuse[0] = 1.0;
	light->diffuse[1] = 1.0;
	light->diffuse[2] = 1.0;
	light->diffuse[3] = 1.0;
	light->specular[0] = 1.0;
	light->specular[1] = 1.0;
	light->specular[2] = 1.0;
	light->specular[3] = 1.0;
	light->SetPos(gal->width_f / 2.0, gal->height_f / 2.0, (gal->height_f + gal->width_f)/4.0);

	gal->ambient.diffuse[0] = 1.0;
	gal->ambient.diffuse[1] = 1.0;
	gal->ambient.diffuse[2] = 1.0;
	gal->ambient.diffuse[3] = 1.0;
	gal->ambient.ambient[0] = 1.0;
	gal->ambient.ambient[1] = 1.0;
	gal->ambient.ambient[2] = 1.0;
	gal->ambient.ambient[3] = 1.0;

	gal->AddLight(light);
	
	CLightEntity *light2;
	light2 = new(CLightEntity);
	light2->diffuse[0] = 1.0;
	light2->diffuse[1] = 1.0;
	light2->diffuse[2] = 1.0;
	light2->diffuse[3] = 1.0;
	light2->specular[0] = 1.0;
	light2->specular[1] = 1.0;
	light2->specular[2] = 1.0;
	light2->specular[3] = 1.0;
	light2->SetPos(gal->width_f / 2, gal->height_f / 2, -(gal->height_f + gal->width_f)/4);
	gal->AddLight(light2);


	player.currentLevel_i = level;
	currentScreen_i = KIT_OUT_SCREEN;
}

void KitOut()
{
	tb.font->FaceSize(20);
	int selection_i = 0;
	int ypos;

	// prepare this
	player.ship_p = new(CShip);
	// shields set by level
	if(player.currentLevel_i < 5)
	{
		player.ship_p->mk1Shield_i = 0;
		player.ship_p->mk2Shield_i = 0;
		player.ship_p->mk3Shield_i = 0;
	}
	else if(player.currentLevel_i < 8)
	{
		player.ship_p->mk1Shield_i = 1;
		player.ship_p->mk2Shield_i = 0;
		player.ship_p->mk3Shield_i = 0;
	}
	else if(player.currentLevel_i < 10)
	{
		player.ship_p->mk1Shield_i = 0;
		player.ship_p->mk2Shield_i = 1;
		player.ship_p->mk3Shield_i = 0;
	}
	else if(player.currentLevel_i < 12)
	{
		player.ship_p->mk1Shield_i = 0;
		player.ship_p->mk2Shield_i = 0;
		player.ship_p->mk3Shield_i = 1;
	}
	else if(player.currentLevel_i < 30) // well, you never know...
	{
		player.ship_p->mk1Shield_i = 0;
		player.ship_p->mk2Shield_i = 1;
		player.ship_p->mk3Shield_i = 1;
	}

	int shipSel=0;
	CDynArray<int> gunSel;
	gunSel.SetSize(gal->shipTypes[gal->shipsAvailToPlayer[shipSel]]->gunPositions.Size());
	for(int gc_i = 0; gc_i< gunSel.Size(); gc_i++)
		gunSel[gc_i] = 0;

	Uint8 *keystate;
	SDL_Delay(500);
	SDL_PumpEvents();
	keystate = SDL_GetKeyState(NULL);
    while(keystate[SDLK_RETURN] == 0)
	{
		SDL_PumpEvents();
		glClear(GL_COLOR_BUFFER_BIT);
		string str;
		// say how many ships / guns there are to choose from
		tb.RenderBG(98, 820, 0.0, 0.25, 0.05, 352);
		tb.RenderBG(98, 800, 0.0, 0.25, 0.05, 352);
		tb.RenderBG(98, 780, 0.0, 0.25, 0.05, 352);
		tb.RenderText(100,820, "Available:");
		str = "Ship Types: " + Cast<string>(gal->shipsAvailToPlayer.Size());
		tb.RenderText(125, 800, str);
		str = "Gun Types: " + Cast<string> (gal->gunTypes.Size()); 
		tb.RenderText(125, 780, str);
		
		// render selection box
		ypos = 700 - (selection_i*20);
		tb.RenderBG(95, ypos, 0.8,0,0, 350);

		// render options		
		tb.RenderText(100, 700, "Ship type: " + gal->shipTypes[gal->shipsAvailToPlayer[shipSel]]->name);
		
		
		// gun 0
		if(gal->shipTypes[gal->shipsAvailToPlayer[shipSel]]->gunPositions.Size() > 0)
		{
			if(gunSel[0] > -1) str = gal->gunTypes[gunSel[0]]->name;
			else str = "Empty";
		}
		else str = "N/A";
		tb.RenderText(100, 680, "Gun1: "+ str);

		// gun 1
		if(gal->shipTypes[gal->shipsAvailToPlayer[shipSel]]->gunPositions.Size() > 1)
		{
			if(gunSel[1] > -1) str = gal->gunTypes[gunSel[1]]->name;
			else str = "Empty";
		}
		else str = "N/A";
		tb.RenderText(100, 660, "Gun2: "+ str);

		// gun 2
		if(gal->shipTypes[gal->shipsAvailToPlayer[shipSel]]->gunPositions.Size() > 2)
		{
			if(gunSel[2] > -1) str = gal->gunTypes[gunSel[2]]->name;
			else str = "Empty";
		}
		else str = "N/A";
		tb.RenderText(100, 640, "Gun3: "+ str);
		
		// gun 3
		if(gal->shipTypes[gal->shipsAvailToPlayer[shipSel]]->gunPositions.Size() > 3)
		{
			if(gunSel[3] > -1) str = gal->gunTypes[gunSel[3]]->name;
			else str = "Empty";
		}
		else str = "N/A";
		tb.RenderText(100, 620, "Gun4: "+ str);

		// gun 4
		if(gal->shipTypes[gal->shipsAvailToPlayer[shipSel]]->gunPositions.Size() > 4)
		{
			if(gunSel[4] > -1) str = gal->gunTypes[gunSel[4]]->name;
			else str = "Empty";
		}
		else str = "N/A";
		tb.RenderText(100, 600, "Gun5: "+ str);

		// gun 4
		if(gal->shipTypes[gal->shipsAvailToPlayer[shipSel]]->gunPositions.Size() > 5)
		{
			if(gunSel[5] > -1) str = gal->gunTypes[gunSel[5]]->name;
			else str = "Empty";
		}
		else str = "N/A";
		tb.RenderText(100, 580, "Gun6: "+ str);

		// gun 5
		if(gal->shipTypes[gal->shipsAvailToPlayer[shipSel]]->gunPositions.Size() > 6)
		{
			if(gunSel[6] > -1) str = gal->gunTypes[gunSel[6]]->name;
			else str = "Empty";
		}
		else str = "N/A";
		tb.RenderText(100, 560, "Gun7: "+ str);

		tb.RenderText(100, 500, "Hull strength:" + Cast<string>(gal->shipTypes[gal->shipsAvailToPlayer[shipSel]]->maxHullHealth_i));
		tb.RenderText(100, 480, "Shield strength:" + Cast<string>((25*player.ship_p->mk1Shield_i) + (45*player.ship_p->mk2Shield_i) + (65*player.ship_p->mk3Shield_i)));
		tb.RenderText(100, 440, "Lives left: " + Cast<string>(player.lives_i));
		tb.RenderText(100, 420, "Score:" + Cast<string>(player.score_i));
		tb.RenderText(100, 120, "Press return to continue");
		tb.RenderText(100, 100, "<- -> change options");
		// and the keys to change the settings
		if(keystate[SDLK_UP]==1)
		{
			if(selection_i > 0)selection_i--; 
			SDL_GL_SwapBuffers();
			SDL_Delay(100);
		}
		else if(keystate[SDLK_DOWN] == 1) 
		{
			if(selection_i < 7)selection_i++;
			SDL_GL_SwapBuffers();
			SDL_Delay(100);
		}
		else if(keystate[SDLK_LEFT] == 1)
		{
			if(selection_i == 0)
			{
				// cycle ships
				if(shipSel > 0) 
				{
					shipSel--;
					gunSel.SetSize(gal->shipTypes[gal->shipsAvailToPlayer[shipSel]]->gunPositions.Size());
					for(int gc_i = 0; gc_i< gunSel.Size(); gc_i++)
						gunSel[gc_i] = 0;
				}
			}
			else
			{
				// cycle the gun in gun position selection-1
				if(gunSel[selection_i-1] > -1) gunSel[selection_i-1]--;
			}
			SDL_GL_SwapBuffers();
			SDL_Delay(100);
		}
		else if(keystate[SDLK_RIGHT] == 1)
		{
			if(selection_i == 0)
			{
				// cycle ships
				if(shipSel < gal->shipsAvailToPlayer.Size()-1) 
				{
					shipSel++;
					gunSel.SetSize(gal->shipTypes[gal->shipsAvailToPlayer[shipSel]]->gunPositions.Size());
					for(int gc_i = 0; gc_i< gunSel.Size(); gc_i++)
						gunSel[gc_i] = 0;
				}
				
			}
			else
			{
				// cycle the gun in gun position selection-1
				if(gunSel[selection_i-1] < gal->gunTypes.Size()-1) gunSel[selection_i-1]++;
			}
			SDL_GL_SwapBuffers();
			SDL_Delay(100);
		}

		else SDL_GL_SwapBuffers();


		
		
	}

	// set up player ship according to settings selected.
	// for now, hard coded..! :)
	
	player.ship_p->shipType_i = gal->shipsAvailToPlayer[shipSel];
	player.ship_p->SetPos(0,0,0);
	player.ship_p->SetVelocity(0,0,0);

	SGun tempGun;
	tempGun.currentCharge_f = 100;
	tempGun.gunType_i = 0;
	tempGun.isActive_b = true;
	tempGun.reloadTime_i = 0;
	for(int gc_i = 0; gc_i < gal->shipTypes[player.ship_p->shipType_i]->gunPositions.Size(); gc_i++)
	{
		tempGun.gunType_i = gunSel[gc_i];
        player.ship_p->guns.AddToEnd(tempGun);
	}


	

	player.ship_p->mesh_p = gal->shipTypes[player.ship_p->shipType_i]->mesh_p;
	player.ship_p->CalculateBoundSph();

	gal->AddShip(player.ship_p);

	gal->shadowsEnabled_b = false;
	gal->lightsEnabled_b = true;
	

	// and into the fight we go!!
	currentScreen_i = IN_LEVEL;
}

bool GameMenu()
{
	tb.font->FaceSize(20);
	int selection_i = 0;
	Uint8 *keystate;
	keystate = SDL_GetKeyState(NULL);
	while(keystate[SDLK_RETURN] == 0)
	{
		glClear(GL_COLOR_BUFFER_BIT);
		SDL_PumpEvents();
		tb.RenderBG(300, 378 + (selection_i*20), 0.8, 0,0,200);

		tb.RenderText(300, 400, "Resume");
		tb.RenderText(300, 380, "Quit");

		if(keystate[SDLK_DOWN] ==1)
		{
			if(selection_i > 0) selection_i--;
			SDL_GL_SwapBuffers();
			SDL_Delay(100);
		}
		else if(keystate[SDLK_UP] ==1)
		{
			if(selection_i < 1) selection_i++;
			SDL_GL_SwapBuffers();
			SDL_Delay(100);
		}
		else SDL_GL_SwapBuffers();
	}
	if(selection_i == 1)
	{
		currentScreen_i = IN_LEVEL;
		return false;
	}
	else
	{
		currentScreen_i = MAIN_MENU;
		return true;
	}
	
}

bool Fly()
{
	glClearColor(0,0,0.1,1);
	float startTime = SDL_GetTicks();
	float currentTime = 0;
	float lastTime = 0;
	int zoom_i = 150;

	int timer = 0;
	bool won = false;
	bool lost = false;

	int renderTime = 0;
	int updateTime = 0;
	int ticks = 0;

	bool end_b = false;

	Uint8 *keystate;
	keystate = SDL_GetKeyState(NULL);
    while(end_b == false)
	{
		SDL_PumpEvents();
		// have we won?? have we lost??
		if((won == true) && ((SDL_GetTicks() - timer) > 2000))
		{
			return true;
		}
		if((lost == true) && ((SDL_GetTicks() - timer) > 2000)) 
		{
			return false;
		}
		
		// update about every 1/70th of a second
		ticks = SDL_GetTicks() - updateTime;
		//if(ticks > 14)
		if(ticks > 12)
		{
			updateTime = SDL_GetTicks();
			if(keystate[SDLK_UP] == 1)
			{
				player.ship_p->FwdThrust();
			}
			if(keystate[SDLK_DOWN] == 1)
			{
				player.ship_p->RvsThrust();
			}
			if(keystate[SDLK_LEFT] == 1)
			{
				player.ship_p->RotateCCW();
			}
			if(keystate[SDLK_RIGHT] == 1)
			{
				player.ship_p->RotateCW();
			}
			if(keystate[SDLK_SPACE] == 1)
			{
				player.ship_p->FireGun();
			}
			if(keystate[SDLK_EQUALS] == 1)
			{
				zoom_i++;
			}
			if(keystate[SDLK_MINUS] == 1)
			{
				zoom_i--;
			}
			if(keystate[SDLK_ESCAPE] == 1)
			{
				// this should take us to the ingame menu, for now, it just ends the round
				if(GameMenu() == true) end_b = true;

			}
		

			for(int ec_i = 0; ec_i < gal->ents.Size(); ec_i++)
				gal->ents[ec_i]->Update();
			gal->Update();
		
			// perform AI on non-player ships
			// also check to see if any with different faction to the player
			// are still alive, and if the player is still alive
			bool enemiesAlive_b = false;
			CShip *shp;
			for(int sc_i = 0; sc_i < gal->ships.Size(); sc_i++)
			{
				shp = (CShip*) gal->ents[gal->ships[sc_i]];
				if((shp != player.ship_p) && (shp->dead_b == false))shp->DoAI();
				if((shp->dead_b == false) && (shp->faction_i != player.ship_p->faction_i))
					enemiesAlive_b = true;
			}

			if((won == false) && (lost == false))
			{
				if(enemiesAlive_b == false) 
				{
					timer = SDL_GetTicks();
					won = true;
					// We've won the level!!
				}
				if(player.ship_p->dead_b == true)
				{
					// We've lost the level >p
					timer = SDL_GetTicks();
					lost = true;
				}
			}

			
		}
		// render about every 1/25th of a second
		ticks = SDL_GetTicks() - renderTime;
		if(ticks > 43) renderTime = SDL_GetTicks();	// getting slow, skip a render
		else if(ticks > 40)
		{	
			renderTime = SDL_GetTicks();
			glColor4f(1.0,1.0,1.0,1.0);

			
			float camPos_af[3];
			int hullDamage, shieldDamage;
			player.ship_p->GetPos(camPos_af);
			hullDamage = player.ship_p->damage_i;
			shieldDamage = (int)player.ship_p->shieldDamage_f;
			FB.scene->camera.SetPos(camPos_af[0], camPos_af[1], camPos_af[2] + zoom_i);
			FB.scene->camera.SetFace(0,0,-1);
			FB.scene->camera.SetUp(0,1,0);
			
			gal->Render();
			glFlush();
			glDisable(GL_BLEND);
			tb.font->FaceSize(2);
			CShip *shp;
			shp = player.ship_p;
			tb.RenderBG(10,100, 1, 0,0,(float) (((float)gal->shipTypes[player.ship_p->shipType_i]->maxHullHealth_i - hullDamage)/gal->shipTypes[player.ship_p->shipType_i]->maxHullHealth_i)*200);
			int mxSldStr_i = (25*shp->mk1Shield_i) + (45*shp->mk2Shield_i) + (65*shp->mk3Shield_i + 1);
			tb.RenderBG(10,120, 0, 0, 1, (float) ((float)(mxSldStr_i - shieldDamage)/(float)mxSldStr_i)*200 +1);

			int ypos = 130;
			for(int gc_i = 0; gc_i < shp->guns.Size(); gc_i++)
			{
				tb.RenderBG(10, (float)ypos, 0, 1, 0, shp->guns[gc_i].currentCharge_f + 1);
				ypos += 10;
			}
			SDL_GL_SwapBuffers();
		}
	}

	currentScreen_i = MAIN_MENU;
	return false;
}






bool Demo()
{

	// prepare the demo.
	gal->FreeAll();
	LoadLevel(-1);
	gal->lightsEnabled_b = true;
	gal->shadowsEnabled_b = false;
	player.ship_p = (CShip*)gal->ents[gal->ships[rand()%gal->ships.Size()]];

	glClearColor(0,0,0.1,1);
	float startTime = SDL_GetTicks();
	float currentTime = 0;
	float lastTime = 0;
	int zoom_i = 200;

	int timer = 0;
	bool won = false;
	bool lost = false;

	int renderTime = 0;
	int updateTime = 0;
	int ticks = 0;

	bool end_b = false;

	Uint8 *keystate;
	keystate = SDL_GetKeyState(NULL);
    while(end_b == false)
	{
		SDL_PumpEvents();
		// have we won?? have we lost??
		if((won == true) && ((SDL_GetTicks() - timer) > 2000))
		{
			return true; 
		}
		if((lost == true) && ((SDL_GetTicks() - timer) > 2000)) 
		{
			return true;	// keep demo going
		}
		
		// update about every 1/70th of a second
		ticks = SDL_GetTicks() - updateTime;
		//if(ticks > 14)
		if(ticks > 12)
		{
			updateTime = SDL_GetTicks();
			
			if(keystate[SDLK_EQUALS] == 1)
			{
				zoom_i++;
			}
			if(keystate[SDLK_MINUS] == 1)
			{
				zoom_i--;
			}
			if(keystate[SDLK_ESCAPE] == 1)
			{
				return false;
			}
			for(int ec_i = 0; ec_i < gal->ents.Size(); ec_i++)
				gal->ents[ec_i]->Update();
			gal->Update();
		
			// perform AI on non-player ships
			// also check to see if any with different faction to the player
			// are still alive, and if the player is still alive
			bool enemiesAlive_b = false;
			CShip *shp;
			for(int sc_i = 0; sc_i < gal->ships.Size(); sc_i++)
			{
				shp = (CShip*) gal->ents[gal->ships[sc_i]];
				if((shp->dead_b == false))shp->DoAI();
				if((shp->dead_b == false) && (shp->faction_i != player.ship_p->faction_i))
					enemiesAlive_b = true;
			}

			if((won == false) && (lost == false))
			{
				if(enemiesAlive_b == false) 
				{
					timer = SDL_GetTicks();
					won = true;
					// We've won the level!!
				}
				if(player.ship_p->dead_b == true)
				{
					// We've lost the level >p
					timer = SDL_GetTicks();
					lost = true;
				}
			}

			
		}
		// render about every 1/25th of a second
		ticks = SDL_GetTicks() - renderTime;
		if(ticks > 40)
		{	
			renderTime = SDL_GetTicks();
			glColor4f(1,1,1,1);

			
			float camPos_af[3];
			int hullDamage, shieldDamage;
			player.ship_p->GetPos(camPos_af);
			hullDamage = player.ship_p->damage_i;
			shieldDamage = (int)player.ship_p->shieldDamage_f;
			FB.scene->camera.SetPos(camPos_af[0], camPos_af[1], camPos_af[2] + zoom_i);
			FB.scene->camera.SetFace(0,0,-1);
			FB.scene->camera.SetUp(0,1,0);
			
			gal->Render();
			glFlush();
			glDisable(GL_BLEND);
			tb.font->FaceSize(2);
			CShip *shp;
			shp = player.ship_p;
			tb.RenderBG(10,100, 1, 0,0,(float) (((float)gal->shipTypes[player.ship_p->shipType_i]->maxHullHealth_i - hullDamage)/gal->shipTypes[player.ship_p->shipType_i]->maxHullHealth_i)*200);
			int mxSldStr_i = (25*shp->mk1Shield_i) + (45*shp->mk2Shield_i) + (65*shp->mk3Shield_i + 1);
			tb.RenderBG(10,120, 0, 0, 1, (float) ((float)(mxSldStr_i - shieldDamage)/(float)mxSldStr_i)*200 +1);

			int ypos = 130;
			for(int gc_i = 0; gc_i < shp->guns.Size(); gc_i++)
			{
				tb.RenderBG(10, (float)ypos, 0, 1, 0, shp->guns[gc_i].currentCharge_f + 1);
				ypos += 10;
			}
			SDL_GL_SwapBuffers();
		}
	}
}













