/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "Ship.h"
#include "Galaxy.h"
#include "FreeBooter.h"
#include "structs.h"
#include "SimpleParticle.h"
#include "GunParticle.h"

CShip::CShip(void)
{
	extern CFreebooter FB;
	CGalaxy *gal = (CGalaxy*)FB.scene;
	cargoQuants.SetSize(gal->cargoTypes.Size());
	for(int c_i = 0; c_i < cargoQuants.Size(); c_i++)
		cargoQuants[c_i] = 0;
	cargoLoad = 0;
	cargoVol = 0;
	shieldDamage_f = 0;
	damage_i = 0;
	// no shields to start with
	mk1Shield_i = 0;
	mk2Shield_i = 0;
	mk3Shield_i = 0;
	inContact_b = false;
	dead_b = false;
	faction_i = 0;
	kills_i = 0;
	SetVelocity(0,0,0);
	SetRotation(0,0,0);
	aiMode_i = AIMD_ROAM;
	aiSubMode_i = 0;
}

CShip::~CShip(void)
{
}


// control functions
void CShip::RotateCW()
{
	extern CFreebooter FB;
	CGalaxy *gal;
	gal = (CGalaxy*) FB.scene;
	float turnSpd = gal->shipTypes[shipType_i]->rotationSpeed_f;
	float zRot = this->ZRot();
	SetRotation( 0, 0, zRot - turnSpd);
}

void CShip::RotateCCW()
{
	extern CFreebooter FB;
	CGalaxy *gal;
	gal = (CGalaxy*) FB.scene;
	float turnSpd = -gal->shipTypes[shipType_i]->rotationSpeed_f;
	float zRot = this->ZRot();
	SetRotation(0,0, zRot - turnSpd);
}

void CShip::FwdThrust()
{
	extern CFreebooter FB;
	CGalaxy *gal;
	gal = (CGalaxy*) FB.scene;
	CVec4f acc_v4f;	// acceleration vector
	float accMag = gal->shipTypes[shipType_i]->fwdThrustPow_f;
	acc_v4f.SetXYZW(0, accMag, 0, 0); 

	// transform to world space
	CMatrix4f tfm;
	tfm.SetIdentity();
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
		glLoadIdentity();
//		glTranslatef(-pos_v4f.X(), -pos_v4f.Y(), -pos_v4f.Z());
		glRotatef(rot_v3f.X(), 1, 0, 0);
		glRotatef(rot_v3f.Y(), 0, 1, 0);
		glRotatef(rot_v3f.Z(), 0, 0, 1);
		glGetFloatv(GL_MODELVIEW_MATRIX, tfm.m_af);
	glPopMatrix();
	tfm.Transpose();
	acc_v4f = acc_v4f.Transform(tfm);
	if(inContact_b == false) vel_v4f = vel_v4f + acc_v4f;

	// add thruster particles
	CVec4f newvel, randVec;
	newvel.SetXYZW(0,0,0,0);
	float xoff, yoff;
	xoff = ((float)(rand()%120))/100;
	yoff = ((float)(rand()%120))/100;
	randVec.SetXYZW(acc_v4f.X() * xoff, acc_v4f.Y() * yoff, acc_v4f.Z(), 0);
	newvel = vel_v4f - (acc_v4f + randVec) * 5;
	float red, green, blue;
	red = 0.5f;
	red   += ((float)(rand()%20))/100;
	green = 0.4f;
	green += ((float)(rand()%20))/100;
	blue  = 0;

	for(int thrc_i = 0; thrc_i < gal->shipTypes[this->shipType_i]->thrPositions.Size(); thrc_i++)
	{
		CVec4f thrpos;
		thrpos = gal->shipTypes[this->shipType_i]->thrPositions[thrc_i].Transform(tfm);
		thrpos = thrpos + pos_v4f;
		
		CSimpleParticle *newPart = NULL;
		try
		{
			newPart = new(CSimpleParticle);
			if(newPart != NULL)
			{
				newPart->SetPos(thrpos.X(), thrpos.Y(), thrpos.Z());
				newPart->SetVelocity(newvel.X(), newvel.Y(), newvel.Z());
				newPart->SetRotVel(0,0,0);
				newPart->life = 20;
				// newPart->SetColour(red, green, blue, 0.25);
				newPart->r_f = red;
				newPart->g_f = green;
				newPart->b_f = blue;
				newPart->a_f = 0.3;
				newPart->hasTexture_b = false;
				thrPartGen.AddParticle(newPart);
			}
		}
		catch(bad_alloc)
		{
			cout << "couldn't allocate particle. - Fwd Thrust" << endl;
		}

	}

	
}

void CShip::RvsThrust()
{
	extern CFreebooter FB;
	CGalaxy *gal;
	gal = (CGalaxy*) FB.scene;
	CVec4f acc_v4f;	// acceleration vector
	float accMag = -gal->shipTypes[shipType_i]->rvsTrustPow_f;
	acc_v4f.SetXYZW(0, accMag, 0, 0); 

	// transform to world space
	CMatrix4f tfm;
	tfm.SetIdentity();
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
		glLoadIdentity();
//		glTranslatef(-pos_v4f.X(), -pos_v4f.Y(), -pos_v4f.Z());
		glRotatef(rot_v3f.X(), 1, 0, 0);
		glRotatef(rot_v3f.Y(), 0, 1, 0);
		glRotatef(rot_v3f.Z(), 0, 0, 1);
		glGetFloatv(GL_MODELVIEW_MATRIX, tfm.m_af);
	glPopMatrix();
	tfm.Transpose();
	acc_v4f = acc_v4f.Transform(tfm);
	vel_v4f = vel_v4f + acc_v4f;
}

void CShip::Update()
{
	extern CFreebooter FB;
	CGalaxy *gal = (CGalaxy*) FB.scene;

	vel_v4f.SetW(0);
	if(vel_v4f.Magnitude() > gal->shipTypes[this->shipType_i]->fwdThrustPow_f * 50)
	{
		vel_v4f.Normalise();
		vel_v4f = vel_v4f * gal->shipTypes[this->shipType_i]->fwdThrustPow_f * 50;
	}
	if(vel_v4f.Magnitude() > 100)
	{
		SetVelocity(0,0,0);
	}

	if(this->dead_b == true)
	{
		Die();
		return;
	}
	if(this->damage_i > gal->shipTypes[this->shipType_i]->maxHullHealth_i)
	{
		this->Die();
		return;
	}
	vel_v4f.SetZ(0.0f);
	pos_v4f.SetZ(0.0f);
	pos_v4f = pos_v4f + vel_v4f;

	// check state of current collision, if any
	if(inContact_b == true)
	{
		// this is fuck arsed. Keep getting entities sent through that don't exist, which
		// is quite a bitch, so let's just fuck it.
		BSBSCollision(contactThis);
		//inContact_b = false;

	}
	// check for a collision with another ship
	else if(inContact_b == false)
	{
		for(int sc_i = 0; sc_i < gal->ships.Size(); sc_i++)
		{
			if((gal->ents[gal->ships[sc_i]] != this) && (((CShip*)(gal->ents[gal->ships[sc_i]]))->dead_b == false))
				if(this->BSBSCollision((CModelEntity*)gal->ents[gal->ships[sc_i]]) == true)
				{
					CVec4f tempVel1;
					CVec4f tempVel2;
					tempVel1 = GetVelocity();
					tempVel2 = gal->ents[gal->ships[sc_i]]->GetVelocity();
					
					tempVel1.SetW(0);
					tempVel2.SetW(0);

					CVec4f entEnt_v4f, negEntEnt;
					entEnt_v4f = gal->ents[gal->ships[sc_i]]->GetPos() - pos_v4f;
					entEnt_v4f.SetW(0);
													
					CVec4f w;
					if(tempVel1.MagSquare() > 0.01)
					{
						//w = tempVel1 * ((tempVel1.Dot(entEnt_v4f)) / tempVel1.MagSquare());
						w = entEnt_v4f * ((tempVel1.Dot(entEnt_v4f)) / entEnt_v4f.MagSquare());
					}
					else w.SetXYZW(0,0,0,0);
					
					tempVel2 = tempVel2 + (w*2.0f);
					tempVel1 = tempVel1 - (w*2.0f);

					// how much will this impact hurt? Magnitude of the difference of velocities
					CVec4f pain_v4f = GetVelocity() - gal->ents[gal->ships[sc_i]]->GetVelocity();
					int pain = pain_v4f.Magnitude() / 2;
					this->damage_i += pain;
					((CShip*)(gal->ents[gal->ships[sc_i]]))->damage_i += pain;


					this->SetVelocity(tempVel1.X(), tempVel1.Y(), 0);
					gal->ents[gal->ships[sc_i]]->SetVelocity(tempVel2.X(), tempVel2.Y(), 0);
				}
		}
		// check for a collision with an inert
		for(int sc_i = 0; sc_i < gal->inerts.Size(); sc_i++)
		{
			//if(gal->ents[gal->inerts[sc_i]] != this)
				if(this->BSBSCollision((CModelEntity*)gal->ents[gal->inerts[sc_i]]) == true)
				{
					CVec4f tempVel1;
					CVec4f tempVel2;
					tempVel1 = GetVelocity();
					tempVel2 = gal->ents[gal->inerts[sc_i]]->GetVelocity();

					tempVel1.SetW(0);
					tempVel2.SetW(0);

					CVec4f entEnt_v4f, negEntEnt;
					entEnt_v4f = gal->ents[gal->inerts[sc_i]]->GetPos() - pos_v4f;
					entEnt_v4f.SetW(0);
													
					CVec4f w;
					if(tempVel1.MagSquare() > 0.01)
					{
						//w = tempVel1 * ((tempVel1.Dot(entEnt_v4f)) / tempVel1.MagSquare());
						w = entEnt_v4f * ((tempVel1.Dot(entEnt_v4f)) / entEnt_v4f.MagSquare());
					}
					else w.SetXYZW(0,0,0,0);
					
					tempVel2 = tempVel2 + w;
					tempVel1 = tempVel1 - w;
					

					// how much will this impact hurt? Magnitude of the difference of velocities
					CVec4f pain_v4f = GetVelocity() - gal->ents[gal->inerts[sc_i]]->GetVelocity();
					float pain = pain_v4f.Magnitude() / 2;
					this->shieldDamage_f += pain;
					

					this->SetVelocity(tempVel1.X(), tempVel1.Y(), 0);
					gal->ents[gal->inerts[sc_i]]->SetVelocity(tempVel2.X(), tempVel2.Y(), 0);
				}
		}
	}

	
	
	thrPartGen.Update();
	gunPartGen.Update();

	for(int gc_i = 0; gc_i < guns.Size(); gc_i++)
	{
		if(guns[gc_i].gunType_i != -1)
		{
			guns[gc_i].currentCharge_f += gal->gunTypes[guns[gc_i].gunType_i]->chargeRate_f;
			if(guns[gc_i].currentCharge_f > gal->gunTypes[guns[gc_i].gunType_i]->maxCharge_f)
				guns[gc_i].currentCharge_f = gal->gunTypes[guns[gc_i].gunType_i]->maxCharge_f;
			guns[gc_i].reloadTime_i -= 1;
		}
	}
	
	if((pos_v4f.X() > gal->width_f) && (vel_v4f.X() > 0)) vel_v4f.SetX(-vel_v4f.X());
	if((pos_v4f.X() < 0) && (vel_v4f.X() < 0)) vel_v4f.SetX(-vel_v4f.X());

	if((pos_v4f.Y() > gal->height_f) && (vel_v4f.Y() > 0)) vel_v4f.SetY(-vel_v4f.Y());
	if((pos_v4f.Y() < 0) && (vel_v4f.Y() < 0)) vel_v4f.SetY(-vel_v4f.Y());

	//charge shields
	if((this->shieldDamage_f > 0)) this->shieldDamage_f -= 0.25;

	// probably more to this (recharge shields, guns, whatever...)

	
}

void CShip::FireGun()
{

	extern CFreebooter FB;
	CGalaxy *gal = (CGalaxy*)FB.scene;

	CVec4f face_v4f;
	face_v4f.SetXYZW(0,1,0,0);

	CMatrix4f tfm;
	tfm.SetIdentity();
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
		glLoadIdentity();
//		glTranslatef(-pos_v4f.X(), -pos_v4f.Y(), -pos_v4f.Z());
		glRotatef(rot_v3f.X(), 1, 0, 0);
		glRotatef(rot_v3f.Y(), 0, 1, 0);
		glRotatef(rot_v3f.Z(), 0, 0, 1);
		glGetFloatv(GL_MODELVIEW_MATRIX, tfm.m_af);
	glPopMatrix();
	tfm.Transpose();

	face_v4f = face_v4f.Transform(tfm);
	
	for(int gc_i = 0; gc_i < this->guns.Size(); gc_i++)
	{
		bool readyToFire_b;
		readyToFire_b = ((guns[gc_i].isActive_b == true) && 
					  (guns[gc_i].currentCharge_f > gal->gunTypes[guns[gc_i].gunType_i]->chargeCost_f) &&
		              (guns[gc_i].reloadTime_i < 1));
		
		// if this is a burst firing cannon
		if(gal->gunTypes[guns[gc_i].gunType_i]->isBurst_b)
		{
			// has the burst come to an end?
			if(guns[gc_i].isBursting_b && !readyToFire_b) guns[gc_i].isBursting_b = false;

			// are we ready to burst again?
			if(!guns[gc_i].isBursting_b && readyToFire_b && (guns[gc_i].currentCharge_f >= gal->gunTypes[guns[gc_i].gunType_i]->minCharge_f))
                guns[gc_i].isBursting_b = true;

			// if gun is not bursting, then it's not firing
			if(!guns[gc_i].isBursting_b) readyToFire_b = false;
		}
		
		


		if(readyToFire_b)
			
		{
			
			
			CVec4f gunPos = gal->shipTypes[this->shipType_i]->gunPositions[gc_i];
			gunPos = gunPos.Transform(tfm);

			gunPos = gunPos + (face_v4f * this->boundSphRadius_f);

			CGunParticle *new_part = NULL;
			try
			{
				new_part = new(CGunParticle);
				if(new_part != NULL)
				{
					new_part->r_f = gal->gunTypes[guns[gc_i].gunType_i]->r_f;
					new_part->g_f = gal->gunTypes[guns[gc_i].gunType_i]->g_f;
					new_part->b_f = gal->gunTypes[guns[gc_i].gunType_i]->b_f;
					new_part->a_f = gal->gunTypes[guns[gc_i].gunType_i]->a_f;
					// new_part->SetColour(r,g,b,a);
					new_part->SetPos(pos_v4f.X() + gunPos.X(), pos_v4f.Y() + gunPos.Y(), 0);
					float spd = gal->gunTypes[guns[gc_i].gunType_i]->speed_f;
					new_part->SetVelocity((face_v4f.X() * spd) + vel_v4f.X(), 
										face_v4f.Y() * spd + vel_v4f.Y(), 0);
					// new_part->life = gal->gunTypes[guns[gc_i].gunType_i]->life;
					new_part->SetRotVel(0,0,3);
					new_part->life = 70;
					new_part->gunType_i = this->guns[gc_i].gunType_i;
					
					new_part->owner = this;
					
					new_part->size_f = gal->gunTypes[guns[gc_i].gunType_i]->partSize_f;
					new_part->boundSphRadius_f = gal->gunTypes[guns[gc_i].gunType_i]->partSize_f;
					new_part->hasBoundSph_b = true;

					this->gunPartGen.AddParticle(new_part);
				}
			}
			catch(bad_alloc)
			{
				cout << "couldn't allocate particle (gun-fire)" << endl;
			}

			this->guns[gc_i].reloadTime_i = gal->gunTypes[guns[gc_i].gunType_i]->fireRate_i;
			this->guns[gc_i].currentCharge_f -= gal->gunTypes[guns[gc_i].gunType_i]->chargeCost_f;
			
		}
	}
}

void CShip::Die()
{
	// for now, just shift the ship to a new random position in the galaxy
	extern CFreebooter FB;
	CGalaxy *gal = (CGalaxy*) FB.scene;
	// and let's have some particles as an explosion, just for fun of course...

	if(aiSubMode_i == 1000)
	{
		
		extern CTextureManager texMan;
		int texInd = texMan.LoadTexture("data/shot.bmp",WIN_BMP);

		dead_b = true;
		int numNewParts = rand()%100 + 100;
		for(int npc_i = 0; npc_i < numNewParts; npc_i++)
		{
			CSimpleParticle *newPart;
			try
			{
				newPart = new(CSimpleParticle);

				newPart->SetPos(X(), Y(), Z());
				newPart->SetVelocity(((float) (rand()%1000)/1000)-0.5,
									((float) (rand()%1000)/1000)-0.5,
									0.0f);
				newPart->life = rand()%70 +12;
				float r = 1.0f;
				//newPart->SetColour(r,r,r,0.7);
				newPart->tex_i = texInd;
				newPart->hasTexture_b = true;
				newPart->r_f = 1.0f;
				newPart->g_f = 1.0f;
				newPart->b_f = 1.0f;
				newPart->a_f = 0.6f;
				newPart->size_f = rand()%3+1;
				gal->galPartGen.AddParticle(newPart);
			}
			catch(bad_alloc)
			{
				cout << "Couldn't allocate particle - AI death" << endl;
			}
			
			aiSubMode_i++;
		}
		
		aiStartTime_i = SDL_GetTicks();
	}
	else
	{
		if((SDL_GetTicks() - aiStartTime_i) > 2000)
		{
			this->SetPos(-400, -400, 0);
			this->SetVelocity(0,0,0);
			this->damage_i = 0;
			this->shieldDamage_f = 0;
			// respawn as a different faction
			// this->faction_i = rand()%gal->factions.Size();
			dead_b = true;
			// also, make sure anything that thinks it's in contact with us
			// no longer has that opinion.
			if(inContact_b)
			{
				contactThis->inContact_b == false;
				contactThis->contactThis = NULL;
				contactThis = NULL;
				inContact_b = false;
			}
		}
	}
}
