/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef FREEBOOTER_SHIP
#define FREEBOOTER_SHIP

#include "ModelEntity.h"
#include "EquipDefines.h"
#include "Dazzle.h"
#include "aiDefines.h"


struct SGun
{
	float  currentCharge_f;	// must be more than minimum needed charge to fire
	int  reloadTime_i;		// must be zero for weapon to fire.
	int  gunType_i;			// what type of gun is this?
	bool isActive_b;		// is this gun switched on?
	bool isBursting_b;		// if this is a burst style gun, this means it's bursting
};

class CShip :
	public CModelEntity
{
public:
	CShip(void);
	~CShip(void);
	
	// for points scoring. How many kills has this ship had?
	int kills_i;

	// pointer to target entity, but how do I know if the target is a ship, base, inert?
	CEntity *target;
	
	// what sort of ship is this?? index the list of ship types of the galaxy
	int shipType_i;

	// What guns are fitted? When the ship is created, this array is set to
	// the number of guns according to the ship type. Each element of the array
	// indexes a gunType in the list for the galaxy, as well as firing status
	// Gun positions are in the ship type.
	CDynArray<SGun> guns;

	// This is the damage that the ship has sustained. Once it equals the total
	// health points for the ship (hull), the ship is dead.
	int damage_i;
	bool dead_b;

	// This indicates the damage inflicted on the shields. Full value, no shields.
	// When shields are down, hull damage occurs. Shields can recharge 
	// though given time, but the hull can not. Some weapons may cause hull damage
	// even if shields are still up...
	float shieldDamage_f;

	// This array is the quantity of each cargo type in the ship.
	CDynArray<int> cargoQuants;

	// This states the current load of the ship
	int cargoLoad;
	int cargoVol;

	// This states the equipment on the ship. Individual bits represent different 
	// equipment types, where a one means it is fit, a zero that it is not.
	unsigned int equipment_ui;

	// This states the faction this ship is allied to
	int faction_i;

	// Shields are separate to other equipment, as a larger number can be fitted.
	// a single shield unit will have less effect on larger ships than smaller ones.
	// More potent shields add more mass, but less than multiple smaller units.
	// however, they cost more and may not fit at all on smaller ships. These
	// values indicate how many of each shield type has been added to the ship.
	int mk1Shield_i;
	int mk2Shield_i;
	int mk3Shield_i;


	// These functions move the ship in some manner
	void RotateCW();
	void RotateCCW();
	void FwdThrust();
	void RvsThrust();

	// This function shoots all guns capable of firing, or that are active
	void FireGun();

	// This function updates a ship's position, steps various recharge states etc..
	void Update();

	// This function is called to deal with when a ship dies
	void Die();

	// The following functions and variable are devoted towards the AI for a ship.
	int aiMode_i;					// one of the AIMD_XXX defines
	int aiSubMode_i;				// some modes may require some extra data
	int aiStartTime_i;				// but maybe we don't want to let a mode last too long
	int aiPrevMode_i;				// the previous ai mode.
	CDynArray<CVec4f> waypoints;	// the destination, and route to that destination.
	
	void DoAI();
	void GenerateRoute();
	void SelectMode();
	int  Roam();
	void Fight();
	void Avoid(CModelEntity *in_Obstacle);	// In dager of collision, avoid!!
	void Formate(); // join formation with target
	// and there are loads more...


	// rewrite the inherited rendering function.
	void Render();

	// the ship's relevent particle generators
	CSimpleParticleGenerator thrPartGen;
	CSimpleParticleGenerator gunPartGen;

};

#endif
