/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ShipLoader.h"
#include <algorithm>
using namespace std;

CShipLoader::CShipLoader(void)
{
}

CShipLoader::~CShipLoader(void)
{
}

CVec4f CShipLoader::ReadPos()
{
	CVec4f out_v4f;
	float one_f, two_f;
	inFile >> one_f >> two_f;
	out_v4f.SetXYZW(one_f, two_f, 0, 1);
	return out_v4f;
}

void CShipLoader::NextLine()
{
	char ch = ' ';
	while((ch != '\n') && (!inFile.eof()))
	{
		inFile.get(ch);
	}
}

int CShipLoader::ReadType()
{
	string s;
	s = ReadToken<string>();
/* argh!! blast the fuckarsed shite things! */
//	transform(s.begin(), s.end(), s.begin(), toupper);
	string::iterator si;
	si = s.begin();
	while(si != s.end())
	{
		*si = toupper(*si);
		si++;
	}
	if(s == "#")	// comment
		return -1;
	else if(s == "NAME")
		return NAME;
	else if(s == "MESH")
		return MESH;
	else if(s == "TEXTURE")
		return TEXTURE;
	else if(s == "MASS")
		return MASS;
	else if(s == "FWDPOW")
		return FWDACC;
	else if(s == "RVSPOW")
		return RVSACC;
	else if(s == "THRPOS")
		return THRPOS;
	else if(s == "GUN")
		return GUN;
	else if(s == "ROTSPD")
		return ROTSPD;
	else if(s == "MAXHEALTH")
		return MAXHEALTH;
	else if(s == "MAXCARGOMASS")
		return MAXCARMAS;
	else if(s == "MAXCARGOVOLUME")
		return MAXCARVOL;
	else if(s == "VALUE")
		return VALUE;
	else return -2;				// unknown/syntax error

}

int CShipLoader::Load(string in_Filename_s, CGalaxy *in_Galaxy)
{
	inFile.clear();
	inFile.open(in_Filename_s.c_str());
	inFile.seekg(0, ios::beg);
	if(!inFile.is_open())
	{
		cout << "Error loading file: " << in_Filename_s << endl;
		exit(1);
	}	
	SShipType *st;
	st = new(SShipType);

	int type_i;
	while(!inFile.eof())
	{
		type_i = ReadType();
		switch(type_i)
		{
		case NAME:
			{
				st->name = ReadToken<string>();
				NextLine();
				break;
			}
		case MESH:
			{
				string str;
				str = ReadToken<string>();
				extern CModelManager modelMan;
				st->mesh_p = modelMan.GetMesh(modelMan.LoadMesh(str, OBJ));
				st->mesh_p->CalcNormals(true);
				st->mesh_p->GenerateDisplayList();
				NextLine();
				break;
			}
		case TEXTURE:
			{
				string str;
				str = ReadToken<string>();
				extern CTextureManager texMan;
				int tex_i = texMan.LoadTexture(str, WIN_BMP);
				st->tex_i.AddToEnd(tex_i);
				NextLine();
				break;
			}
		case MASS:
			{
				st->mass_f = ReadToken<float>();
				NextLine();
				break;
			}
		case THRPOS:
			{
				st->thrPositions.AddToEnd(ReadPos());
				NextLine();
				break;
			}
		case FWDACC:
			{
				st->fwdThrustPow_f = ReadToken<float>();
				NextLine();
				break;
			}
		case RVSACC:
			{
				st->rvsTrustPow_f = ReadToken<float>();
				NextLine();
				break;
			}
		case GUN:
			{
				st->gunPositions.AddToEnd(ReadPos());
				NextLine();
				break;
			}
		case ROTSPD:
			{
				st->rotationSpeed_f = ReadToken<float> ();
				NextLine();
				break;
			}
		case MAXHEALTH:
			{
				st->maxHullHealth_i = ReadToken<int> ();
				NextLine();
				break;
			}
		case MAXCARMAS:
			{
				st->maxCargoMass_i = ReadToken<int> ();
				NextLine();
				break;
			}
		case MAXCARVOL:
			{
				st->maxCargoVol_i = ReadToken<int> ();
				NextLine();
				break;
			}
		case VALUE:
			{
				st->value_i = ReadToken<int>();
				NextLine();
				break;
			}
		case -1:	// comment
			{
				NextLine();
				break;
			}
		case -2:	// syntax error?
			{
				NextLine();
				break;
			}		
		}
	}
	in_Galaxy->shipTypes.AddToEnd(st);
	return 0;
}
