/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*
ship rendering function
*/

#include "Ship.h"
#include "Galaxy.h"
#include "aiDefines.h"

void CShip::Render()
{// Set opengl matrix to entity position
	
	if(dead_b == true) return;
	glPushMatrix();
	
	glTranslatef(pos_v4f.X(), pos_v4f.Y(), pos_v4f.Z());
	glRotatef(rot_v3f.X(), 1, 0, 0);
    glRotatef(rot_v3f.Y(), 0, 1, 0);
    glRotatef(rot_v3f.Z(), 0, 0, 1);
	

	extern CTextureManager texMan;
	glEnable(GL_TEXTURE_2D);
	GLuint texind;
	extern CFreebooter FB;
	CGalaxy *gal;
	gal = (CGalaxy*) FB.scene;

	// figure out which texture to apply
	if(faction_i > gal->shipTypes[shipType_i]->tex_i.Size()-1)
		texind = gal->shipTypes[shipType_i]->tex_i[0];
	else
		texind = gal->shipTypes[shipType_i]->tex_i[faction_i];
	
	glBindTexture(GL_TEXTURE_2D, texMan.textures[texind].id_ui);
	
	// render the entity
	extern CModelManager modelMan;
	CMesh *mesh;
	mesh = gal->shipTypes[shipType_i]->mesh_p;

	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);

	if(mesh->has_displayList)
	{
		glCallList(mesh->dispList);
	}
	else
	{
		// cycle faces
		for(int fc_i = 0; fc_i < mesh->faces.Size(); fc_i++)
		{
			CFace *face;
			face = mesh->faces[fc_i];
			// draw face
			glBegin(GL_POLYGON);
			// cycle points of face
			for(int pc_i = 0; pc_i < face->p.Size(); pc_i++)
			{
				if(mesh->has_normals)
				{
					glNormal3f(mesh->normals[face->n[pc_i]].X(),
							mesh->normals[face->n[pc_i]].Y(),
							mesh->normals[face->n[pc_i]].Z());
				}
				// tex-coords
				if(mesh->has_uvpoints)
				{	
					glTexCoord2f(mesh->uvpoints[face->uv[pc_i]].X(),
								mesh->uvpoints[face->uv[pc_i]].Y());
				}
				if(mesh->has_colors)
				{
				}
				// anything else?
				// and draw it
				glVertex3f(mesh->points[face->p[pc_i]].X(),
						mesh->points[face->p[pc_i]].Y(),
						mesh->points[face->p[pc_i]].Z());
	            
			}
			glEnd();
		}
	}
	
	// draw subentities
	for(int sec_i = 0; sec_i < sub_ents.Size(); sec_i++)
	{
		sub_ents[sec_i]->Render();
	}
    
	// reset OpenGL matrix to value prior to rendering this entity
	glPopMatrix();

	// render ship related particles
	gunPartGen.Render();
	thrPartGen.Render();

	

	/*
	// draw a line from the ship to its first waypoint
	glDisable(GL_LIGHTING);
	glDisable(GL_BLEND);
	glLineWidth(3);
	glColor4f(1,0,0,1);
	CVec4f origin, end;
	origin = pos_v4f;
	for(int wc_i = waypoints.Size()-1; wc_i > -1; wc_i--)
	{
		end = waypoints[wc_i];
		glBegin(GL_LINES);
			glVertex3f(origin.X(), origin.Y(), origin.Z());
			glVertex3f(end.X(), end.Y(), end.Z());
		glEnd();
		origin = end;
	}
	
	glEnable(GL_LIGHTING);
	//*/
}
