/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DYN_ARRAY
#define DYN_ARRAY

#include <cstdlib>
#include <cstdio>
using namespace std;

// template class for a dynamic array
template<class T> class CDynArray
{
	T *elements;
	int numberOfElements;

public:
	CDynArray(void);
	~CDynArray(void);

	// Set the size of the array to the new size
	bool SetSize(int in_NewSize);
	// Add a new element to the end of the array
	bool AddToEnd(T in_NewElement);
	// Return number of elements in array
	int Size(void);
	// Return a pointer to the start of the array proper
	T	*GetPointerToFirst();
	// delete all contents of array
	void Clear();
	// Array Access
	T &operator[](int i) {return elements[i];}
};

template<class T> CDynArray<T>::CDynArray(void)
{
	numberOfElements = 0;
	elements = NULL;
}

template<class T> 
CDynArray<T>::~CDynArray(void)
{
	//Clear();
}

template<class T> bool CDynArray<T>::SetSize(int in_NewSize)
{
	if(elements == NULL)
	{
		elements = (T*)malloc(sizeof(T) * in_NewSize);
		if(elements == NULL)
		{
			fprintf(stderr, "Error allocating memory for call to CDynArray::SetSize(%d)\n out of memory?\n", in_NewSize);
			return false;	// error allocating new memory for array
		}
		else 
		{
			numberOfElements = in_NewSize;
			return true;
		}
	}
	else
	{
		T *newElements;
		newElements = (T*)realloc(elements, sizeof(T) * in_NewSize);
		if(newElements == NULL)
		{
			fprintf(stderr, "Error reallocating memory for call to CDynArray::SetSize(%d)\n out of memory?\n", in_NewSize);
			return false;	// error allocating new memory for array
		}
		else
		{
			elements = newElements;
			numberOfElements = in_NewSize;
			return true;
		}
	}
}

template<class T> bool CDynArray<T>::AddToEnd(T in_NewElement)
{
	// increase array size by one
	if(SetSize(numberOfElements+1))
	{
		elements[numberOfElements-1] = in_NewElement;
		return true;
	}
	else
	{
		fprintf(stderr, "Called from CDynArray::AddToEnd");
		return false;
	}
}

template<class T> int CDynArray<T>::Size(void)
{
	return numberOfElements;
}

template<class T> T *CDynArray<T>::GetPointerToFirst()
{
	return elements;
}

template<class T> void CDynArray<T>::Clear()
{
	if(elements != NULL) free(elements);
	elements = NULL;
	numberOfElements = 0;
}



#endif