/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SCENE_ENTITY_BASE
#define SCENE_ENTITY_BASE
/********************************************************************
 *							     Dazzle								*
 *							Base entity class						*
 *																	*
 *							   Murray Evans							*
 ********************************************************************/
#include "Matlib.h"		// header file for matrix library
#include "DynArray.h"
#include <string>
#include <vector>
using namespace std;
class CEntity
{
public:
	CEntity(void);
	virtual ~CEntity(void);
	virtual void PrepareForDestruction();
protected:
	CVec4f				pos_v4f;		// position vector
	CVec3f				rot_v3f;		// axis rotations (orientation)	
	CVec4f				vel_v4f;		// Velocity vector
	CVec3f				rv_v3f;			// axis velocities
	CDynArray<CEntity*>	sub_ents;		// sub-entities
	
public:
	// Set translation from world origin
	void SetPos(float in_X_f, float in_Y_f, float in_Z_f);
	// Set the velocity vector using x, y, z
	void SetVelocity(float in_Xvel_f, float in_Yvel_f, float in_Zvel_f);
	// Set the velocity vector using a vec4f
	void SetPos(CVec4f in_v4f);

	// Set rotation of entity by axis rotations (x,y,z)
	void SetRotation(float in_Xrot_f, float in_Yrot_f, float in_Zrot_f);
	// Set rotational velocity of entity by axis (x,y,z)
	void SetRotVel(float in_Xrot_f, float in_Yrot_f, float in_Zrot_f);
	CVec3f GetRotVel();
	// return x position
	float X(void);
	// return y position
	float Y(void);
	// return z position
	float Z(void);
	// return x rotation
	float XRot(void);
	// return y rotation
	float YRot(void);
	// return z rotation
	float ZRot(void);
	// put position data into the array pointed to by io_paf. Array should be float[3] for (x,y,z)
	void GetPos(float* io_paf);
	// return vec4f of position data
	CVec4f GetPos();
	// get velocity vector
	CVec4f GetVelocity();
	// Rendering function. Mostly it will be more evolved entities which actually have anythin happen here
	virtual void Render(void);
	// an update function
	virtual void Update(void);
};


#endif
