/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "Collision.h"
#include "ModelEntity.h"
#include "ModelManager.h"
#include "GL/gl.h"

// bounding sphere-bounding sphere collision detection.
bool CModelEntity::BSBSCollision(CModelEntity *in_Entity)
{
	
	// if the length of the vector between the two entities is 
	// less than the length of their two bounding sphere radii
	// then the BS's intersect, return true, else false.
	// Similarly, if the squared length of the vector is less than 
	// the squared length of the two BSR's... avoids sqrt.

	if(!hasBoundSph_b) CalculateBoundSph();
	if(!in_Entity->hasBoundSph_b) in_Entity->CalculateBoundSph();

	if((abs(X() - in_Entity->X()) < 2 * in_Entity->boundSphRadius_f) &&
	   (abs(Y() - in_Entity->Y()) < 2 * in_Entity->boundSphRadius_f) &&
	   (abs(Z() - in_Entity->Z()) < 2 * in_Entity->boundSphRadius_f))
	{
		CVec4f entEnt_v4f;
		float  magEntEntSQ_f;
		float  radAddRad_f;
		float  radAddRadSQ_f;

		
		entEnt_v4f    = pos_v4f - in_Entity->GetPos();
		entEnt_v4f.SetW(0);
		magEntEntSQ_f = entEnt_v4f.MagSquare();
		if(inContact_b == false)
		    radAddRad_f   = in_Entity->boundSphRadius_f + boundSphRadius_f;
		else
		    radAddRad_f   = (in_Entity->boundSphRadius_f*1.25) + (boundSphRadius_f*1.25);
		radAddRadSQ_f = radAddRad_f * radAddRad_f;

		if(radAddRadSQ_f > magEntEntSQ_f)
		{
			inContact_b = true;
			contactThis = in_Entity;
			in_Entity->inContact_b = true;
			in_Entity->contactThis = this;
			return true;
		}
	}
	inContact_b = false;
	contactThis = NULL;
	return false;
}


// bounding sphere of this colliding with mesh of in_Entity
bool CModelEntity::BSMeshCollision(CModelEntity *in_Entity, SEntityCollision *out_Col)
{
	// get origin of this in local space of in_Entity
	CVec4f thisLocal_v4f;
	CMatrix4f tfm;
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
		glLoadIdentity();
			
		glTranslatef(-in_Entity->X(), -in_Entity->Y(), -in_Entity->Z());
		glRotatef(-in_Entity->XRot(), 1, 0, 0);
		glRotatef(-in_Entity->YRot(), 0, 1, 0);
		glRotatef(-in_Entity->ZRot(), 0, 0, 1);

		glGetFloatv(GL_MODELVIEW_MATRIX, tfm.m_af);
	glPopMatrix();
	tfm.Transpose();
	thisLocal_v4f = pos_v4f.Transform(tfm);

	// collide against entity, then sub-entities
	// Ignoring sub entities for now. Feeling like I want something to show, not lots
	// and lots of code doing nothing...
	SMeshCollision mcol;
	if(in_Entity->mesh_p == NULL)
	{
		extern CModelManager modelMan;
		in_Entity->mesh_p = modelMan.GetMesh(in_Entity->mesh_i);
	}
	if(!in_Entity->mesh_p->CollideMeshSphere(thisLocal_v4f, boundSphRadius_f, &mcol))
	{
		inContact_b = false;
		return false;
	}
	else
	{
		out_Col->entity_p = (CEntity*)in_Entity;
		out_Col->mCol = mcol;
		out_Col->subent_ai.Clear();
		inContact_b = true;
		return true;
	}

}

// bounding sphere of this colliding with vector in world space
// pass in vector start point and vector end point
bool CModelEntity::BSVectCollision(CVec4f in_VectSP, CVec4f in_VectEP)
{
	// this just says if there is an intersection, not where the intersection is.

	// vector from vector SP to BSorigin
	CVec4f vsptoBSo;
	
	// vector from vector SP to vector EP
	CVec4f vsptoVep;

	vsptoBSo = this->pos_v4f - in_VectSP;
	vsptoVep = in_VectEP - in_VectSP;
	float h = vsptoBSo.Magnitude();
	// angle between vectors
	float theta;
	vsptoBSo.Normalise();
	vsptoVep.Normalise();
	theta = acos(vsptoBSo.Dot(vsptoVep));
    
	// perpendicular distance from BS origin to a point on the line
	float d;
	d = h * sin(theta);

	if(d < this->boundSphRadius_f) return true;
	else return false;
}
