/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "Matrix3f.h"

/************************************************************************
 *						3D matrix class - Matlib						*
 *																		*
 *						    Murray Evans 2003							*
 ************************************************************************/
CMatrix3f::CMatrix3f(void)
{
}

CMatrix3f::CMatrix3f(float in_00, float in_10, float in_20,
			  float in_01, float in_11, float in_21,
			  float in_02, float in_12, float in_22)
{
	m_af[0] = in_00;
	m_af[1] = in_10;
	m_af[2] = in_20;
	m_af[3] = in_01;
	m_af[4] = in_11;
	m_af[5] = in_21;
	m_af[6] = in_02;
	m_af[7] = in_12;
	m_af[8] = in_22;
}

CMatrix3f::CMatrix3f(float in_M_af[9])
{
	for(int c_i = 0; c_i < 9; c_i++)
	{
		m_af[c_i] = in_M_af[c_i];
	}
}

CMatrix3f::~CMatrix3f(void)
{
}

void CMatrix3f::SetIdentity()
{
	m_af[0] = 1; m_af[1] = 0; m_af[2] = 0;
	m_af[3] = 0; m_af[4] = 1; m_af[5] = 0;
	m_af[6] = 0; m_af[7] = 0; m_af[8] = 1;
}

CMatrix3f CMatrix3f::PreMult(CMatrix3f in_m3f)
{
	int r_i, c_i;
    float sum_f = 0;
	CMatrix3f out_m3f;

    for(int x_i = 0; x_i < 9; x_i++)
    {
        r_i = x_i / 3;  c_i = x_i % 3;

        for(int y_i = 0; y_i < 3; y_i++)
        {
			sum_f = sum_f + in_m3f.m_af[(r_i*3) + y_i] * m_af[(y_i*3)+c_i];
        }

        out_m3f.m_af[(r_i*3)+c_i] = sum_f;
        sum_f = 0;
    }
	return out_m3f;
}

CMatrix3f CMatrix3f::PostMult(CMatrix3f in_m3f)
{
	int r_i, c_i;
    float sum_f = 0;
	CMatrix3f out_m3f;

    for(int x_i = 0; x_i < 9; x_i++)
    {
        r_i = x_i / 3;  c_i = x_i % 3;

        for(int y_i = 0; y_i < 3; y_i++)
        {
			sum_f = sum_f +  m_af[(r_i*3) + y_i] * in_m3f.m_af[(y_i*3)+c_i];
        }

        out_m3f.m_af[(r_i*3)+c_i] = sum_f;
        sum_f = 0;
    }
	return out_m3f;
}


// This is very very likely to be wrong and not work, and also stupidly slow.
// As this is the basis however for a graphics engine, m3f's aren't that likely
// to be inverted, it's more the homogeneous m4f's that will be. :)
void CMatrix3f::Invert()
{
	// errmm, remember how to do this???
	// Gauss-Jordan method
	CMatrix3f inv_m3f;
	inv_m3f.SetIdentity();

	// first step: make bottom two rows have 0 in the first column
	float x_f, y_f;
	x_f = -(m_af[0]/m_af[3]);
	y_f = -(m_af[0]/m_af[6]);
	// row2 = row1 + row2 * x_f
	m_af[3] = 0;	// m_af[3] = m_af[0] + (m_af[3] * x_f);
	m_af[4] = m_af[1] + (m_af[4] * x_f);
	m_af[5] = m_af[2] + (m_af[5] * x_f);
	inv_m3f.m_af[3] = inv_m3f.m_af[0] + (inv_m3f.m_af[3] * x_f);
	inv_m3f.m_af[4] = inv_m3f.m_af[1] + (inv_m3f.m_af[4] * x_f);
	inv_m3f.m_af[5] = inv_m3f.m_af[2] + (inv_m3f.m_af[5] * x_f);
	// row3 = row1 + row3 * y_f
	m_af[6] = 0;	// m_af[6] = m_af[0] + (m_af[6] * y_f);
	m_af[7] = m_af[1] + (m_af[7] * y_f);
	m_af[8] = m_af[2] + (m_af[8] * y_f);
	inv_m3f.m_af[6] = inv_m3f.m_af[0] + (inv_m3f.m_af[6] * y_f);
	inv_m3f.m_af[7] = inv_m3f.m_af[1] + (inv_m3f.m_af[7] * y_f);
	inv_m3f.m_af[8] = inv_m3f.m_af[2] + (inv_m3f.m_af[8] * y_f);
	

	// second step: make bottom row have 0 in second column
	y_f = -(m_af[4]/m_af[7]);
	// row3 = row2 + row3*y_f
	m_af[6] = 0;	// m_af[3] + (m_af[6] * y_f);
	m_af[7] = 0;	// m_af[4] + (m_af[7] * y_f);
	m_af[8] = m_af[5] + (m_af[8] * y_f);
	inv_m3f.m_af[6] = inv_m3f.m_af[3] + (inv_m3f.m_af[6] * y_f);
	inv_m3f.m_af[7] = inv_m3f.m_af[4] + (inv_m3f.m_af[7] * y_f);
	inv_m3f.m_af[8] = inv_m3f.m_af[5] + (inv_m3f.m_af[8] * y_f);

	// third step: make top two rows have 0 in third column
	x_f = -(m_af[8]/m_af[2]);
	y_f = -(m_af[5]/m_af[2]);
	// row1 = row3 + row1*x_f
	m_af[0] = (m_af[0] * x_f) + m_af[6];
	m_af[1] = (m_af[1] * x_f) + m_af[7];
	m_af[2] = (m_af[2] * x_f) + m_af[8];
	inv_m3f.m_af[0] = (inv_m3f.m_af[0] * x_f) + inv_m3f.m_af[6]; 
	inv_m3f.m_af[1] = (inv_m3f.m_af[1] * x_f) + inv_m3f.m_af[7]; 
	inv_m3f.m_af[2] = (inv_m3f.m_af[2] * x_f) + inv_m3f.m_af[8];
	// row2 = row3 + row2*y_f
	m_af[3] = (m_af[3] * y_f) + m_af[6];
	m_af[4] = (m_af[4] * y_f) + m_af[7];
	m_af[5] = (m_af[5] * y_f) + m_af[8];
	inv_m3f.m_af[3] = (inv_m3f.m_af[3] * x_f) + inv_m3f.m_af[6]; 
	inv_m3f.m_af[4] = (inv_m3f.m_af[4] * x_f) + inv_m3f.m_af[7]; 
	inv_m3f.m_af[5] = (inv_m3f.m_af[5] * x_f) + inv_m3f.m_af[8];

	// fourth step: make top row have 0 in second column
	x_f = -(m_af[4]/m_af[1]);
	// row1 = row2 + row1*y_f
	m_af[0] = (m_af[0] * x_f) + m_af[3];
	m_af[1] = (m_af[1] * x_f) + m_af[4];
	m_af[2] = (m_af[2] * x_f) + m_af[5];
	inv_m3f.m_af[0] = (inv_m3f.m_af[0] * x_f) + inv_m3f.m_af[3]; 
	inv_m3f.m_af[1] = (inv_m3f.m_af[1] * x_f) + inv_m3f.m_af[4]; 
	inv_m3f.m_af[2] = (inv_m3f.m_af[2] * x_f) + inv_m3f.m_af[5];

	// final step: make sure m_af is now identity (1's on diagonals)
	// row1 = row1 * 1/m_af[0];
	x_f = 1/m_af[0];
/*	m_af[0] *= x_i;
	m_af[1] *= x_i;
	m_af[2] *= x_i;*/
	inv_m3f.m_af[0] *= x_f;
	inv_m3f.m_af[1] *= x_f;
	inv_m3f.m_af[2] *= x_f;
	// row2 = row2 * 1/m_af[4];
	x_f = 1/m_af[4];
/*	m_af[3] *= x_i;
	m_af[4] *= x_i;
	m_af[5] *= x_i;*/
	inv_m3f.m_af[3] *= x_f;
	inv_m3f.m_af[4] *= x_f;
	inv_m3f.m_af[5] *= x_f;
	// row3 = row3 * 1/m_af[8];
	x_f = 1/m_af[8];
/*	m_af[6] *= x_i;
	m_af[7] *= x_i;
	m_af[8] *= x_i; // don't need to do this */
	inv_m3f.m_af[6] *= x_f;
	inv_m3f.m_af[7] *= x_f;
	inv_m3f.m_af[8] *= x_f;

	for(int c_i = 0; c_i < 9; c_i++)
	{
		m_af[c_i] = inv_m3f.m_af[c_i];
	}
	// done ??

}

void CMatrix3f::Transpose()
{
	CMatrix3f trn_m3f;
	for(int c_i = 0; c_i < 9; c_i++)
	{
		trn_m3f.m_af[c_i] = m_af[c_i];
	}
	m_af[1] = trn_m3f.m_af[3];
	m_af[2] = trn_m3f.m_af[6];
	m_af[3] = trn_m3f.m_af[1];
	m_af[5] = trn_m3f.m_af[7];
	m_af[6] = trn_m3f.m_af[2];
	m_af[7] = trn_m3f.m_af[5];
}

CMatrix3f CMatrix3f::operator+(CMatrix3f& in_m3f)
{
	CMatrix3f out_m3f;
	for(int c_i = 0; c_i < 9; c_i ++)
	{
		out_m3f.m_af[c_i] = m_af[c_i] + in_m3f.m_af[c_i];
	}
	return out_m3f;
}

CMatrix3f CMatrix3f::operator-(CMatrix3f& in_m3f)
{
	CMatrix3f out_m3f;
	for(int c_i = 0; c_i < 9; c_i ++)
	{
		out_m3f.m_af[c_i] = m_af[c_i] - in_m3f.m_af[c_i];
	}
	return out_m3f;
}


string CMatrix3f::WriteOut()
{
	string out_s;
	out_s = "| " + Cast<string>(m_af[0]) + "\t" + Cast<string>(m_af[1]) + "\t" + Cast<string>(m_af[2]) + "\t|\n" +
		    "| " + Cast<string>(m_af[3]) + "\t" + Cast<string>(m_af[4]) + "\t" + Cast<string>(m_af[5]) + "\t|\n" +
			"| " + Cast<string>(m_af[6]) + "\t" + Cast<string>(m_af[7]) + "\t" + Cast<string>(m_af[8]) + "\t|\n";
	return out_s;
}
