/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "Matrix4f.h"


/************************************************************************
 *					3D Homogeneous matrix class - Matlib				*
 *																		*
 *						    Murray Evans 2003							*
 ************************************************************************/

CMatrix4f::CMatrix4f(void)
{
}
CMatrix4f::CMatrix4f(float in_00, float in_10, float in_20, float in_30,
				     float in_01, float in_11, float in_21, float in_31,
				     float in_02, float in_12, float in_22, float in_32,
				     float in_03, float in_13, float in_23, float in_33)
{
	m_af[0] = in_00;
	m_af[1] = in_10;
	m_af[2] = in_20;
	m_af[3] = in_30;
	
	m_af[4] = in_01;
	m_af[5] = in_11;
	m_af[6] = in_21;
	m_af[7] = in_31;

	m_af[8] = in_02;
	m_af[9] = in_12;
	m_af[10]= in_22;
	m_af[11]= in_32;

	m_af[12]= in_03;
	m_af[13]= in_13;
	m_af[14]= in_23;
	m_af[15]= in_33;
}

CMatrix4f::CMatrix4f(float in_M_af[16])
{
	for(int c_i = 0; c_i < 16; c_i++)
	{
		m_af[c_i] = in_M_af[c_i];
	}
}


CMatrix4f::~CMatrix4f(void)
{
}

void CMatrix4f::SetIdentity()
{
	m_af[0] = 1; m_af[1] = 0; m_af[2] = 0; m_af[3] = 0;
	m_af[4] = 0; m_af[5] = 1; m_af[6] = 0; m_af[7] = 0;
	m_af[8] = 0; m_af[9] = 0; m_af[10] = 1;m_af[11]= 0;
	m_af[12] = 0; m_af[13] = 0; m_af[14] = 0;m_af[15]= 1;
}

CMatrix4f CMatrix4f::PreMult(CMatrix4f in_m4f)
{
	int r_i, c_i;
    float sum_f = 0;
	CMatrix4f out_m4f;

    for(int x_i = 0; x_i < 16; x_i++)
    {
        r_i = x_i / 4;  c_i = x_i % 4;

        for(int y_i = 0; y_i < 16; y_i++)
        {
			sum_f = sum_f + in_m4f.m_af[(r_i*4) + y_i] * m_af[(y_i*4)+c_i];
        }

        out_m4f.m_af[(r_i*4)+c_i] = sum_f;
        sum_f = 0;
    }
	return out_m4f;
}

CMatrix4f CMatrix4f::PostMult(CMatrix4f in_m4f)
{
	int r_i, c_i;
    float sum_f = 0;
	CMatrix4f out_m4f;

    for(int x_i = 0; x_i < 4; x_i++)
    {
        r_i = x_i / 4;  c_i = x_i % 4;

        for(int y_i = 0; y_i < 4; y_i++)
        {
			sum_f = sum_f +  m_af[(r_i*4) + y_i] * in_m4f.m_af[(y_i*4)+c_i];
        }

        out_m4f.m_af[(r_i*4)+c_i] = sum_f;
        sum_f = 0;
    }
	return out_m4f;
}

// assume that we're inverting a homogeneous matrix, thus avoiding Gauss-Jordan inverse
// or worse still, Cramer's rule
void CMatrix4f::Invert()
{
	CMatrix4f inv_m4f;
	//Transpose rotation
	inv_m4f.m_af[0]  = m_af[0];
	inv_m4f.m_af[1]  = m_af[4];
	inv_m4f.m_af[2]  = m_af[8];
	inv_m4f.m_af[4]  = m_af[1];
	inv_m4f.m_af[5]  = m_af[5];
	inv_m4f.m_af[6]  = m_af[9];
	inv_m4f.m_af[8]  = m_af[2];
	inv_m4f.m_af[9]  = m_af[6];
	inv_m4f.m_af[10] = m_af[10];
   

    //Negate and rotate translation
    float tx_f = -m_af[3];
    float ty_f = -m_af[7];
    float tz_f = -m_af[11];

	inv_m4f.m_af[3]  = tx_f * m_af[0] + ty_f * m_af[1] + tz_f * m_af[2];
    inv_m4f.m_af[7]  = tx_f * m_af[4] + ty_f * m_af[5] + tz_f * m_af[6];
    inv_m4f.m_af[11] = tx_f * m_af[8] + ty_f * m_af[9] + tz_f * m_af[10];

    //Fill the bottom row with 0,0,0,1
    inv_m4f.m_af[12] = 0.0f;
    inv_m4f.m_af[13] = 0.0f;
    inv_m4f.m_af[14] = 0.0f;
    inv_m4f.m_af[15] = 1.0f;

	for(int i = 0; i < 16; i++)
		m_af[i] = inv_m4f.m_af[i];
}

void CMatrix4f::Transpose()
{
	CMatrix4f trn_m4f;
	for(int c_i = 0; c_i < 16; c_i++)
	{
		trn_m4f.m_af[c_i] = m_af[c_i];
	}

	m_af[0]  = trn_m4f.m_af[0];
	m_af[1]  = trn_m4f.m_af[4];
	m_af[2]  = trn_m4f.m_af[8];
	m_af[3]  = trn_m4f.m_af[12];
	
	m_af[4]  = trn_m4f.m_af[1];
	m_af[5]  = trn_m4f.m_af[5];
	m_af[6]  = trn_m4f.m_af[9];
	m_af[7]  = trn_m4f.m_af[13];

	m_af[8]  = trn_m4f.m_af[2];
	m_af[9]  = trn_m4f.m_af[6];
	m_af[10] = trn_m4f.m_af[10];
	m_af[11] = trn_m4f.m_af[14];

	m_af[12] = trn_m4f.m_af[3];
	m_af[13] = trn_m4f.m_af[7];
	m_af[14] = trn_m4f.m_af[11];
	m_af[15] = trn_m4f.m_af[15];
	
}

CMatrix4f CMatrix4f::operator+(CMatrix4f& in_m4f)
{
	CMatrix4f out_m4f;
	for(int c_i = 0; c_i < 16; c_i ++)
	{
		out_m4f.m_af[c_i] = m_af[c_i] + in_m4f.m_af[c_i];
	}
	return out_m4f;
}

CMatrix4f CMatrix4f::operator-(CMatrix4f& in_m4f)
{
	CMatrix4f out_m4f;
	for(int c_i = 0; c_i < 16; c_i ++)
	{
		out_m4f.m_af[c_i] = m_af[c_i] - in_m4f.m_af[c_i];
	}
	return out_m4f;
}


string CMatrix4f::WriteOut()
{
	string out_s;
	out_s = "| " + Cast<string>(m_af[0]) + "\t" + Cast<string>(m_af[1]) + "\t" + Cast<string>(m_af[2]) + "\t" + Cast<string>(m_af[7]) + "\t|\n" +
		    "| " + Cast<string>(m_af[4]) + "\t" + Cast<string>(m_af[5]) + "\t" + Cast<string>(m_af[6]) + "\t" + Cast<string>(m_af[11]) + "\t|\n" +
			"| " + Cast<string>(m_af[8]) + "\t" + Cast<string>(m_af[9]) + "\t" + Cast<string>(m_af[10]) + "\t" + Cast<string>(m_af[15]) + "\t|\n" +
			"| " + Cast<string>(m_af[12]) + "\t" + Cast<string>(m_af[14]) + "\t" + Cast<string>(m_af[14]) + "\t" + Cast<string>(m_af[3]) + "\t|\n";
	return out_s;
}
