/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "Mesh.h"
/********************************************************************
 *								 Dazzle								*
 *							 Mesh data class						*
 *																	*
 *							   Murray Evans							*
 ********************************************************************/

CMesh::CMesh(void)
{
	this->has_displayList = false;
}

CMesh::~CMesh(void)
{
}

void CMesh::CalcNormals(bool in_Force_b)
{
	CalcFaceNormals(in_Force_b);
	if((!has_normals) || (in_Force_b))
	{
		normals.Clear();
		// make as many blank normals as there are verts
        for(int pc_i = 0; pc_i < points.Size(); pc_i++)
       	{
       		CVec4f temp_v4f;
       		temp_v4f.SetXYZW(0,0,0,0);
         	normals.AddToEnd(temp_v4f);
       	}
        // and give each face as many normal indices as it has point indices
        for(int fc_i = 0; fc_i < faces.Size(); fc_i++)
        {
			faces[fc_i]->n.Clear();
			for(int pc_i = 0; pc_i < faces[fc_i]->p.Size(); pc_i++)
			{
				faces[fc_i]->n.AddToEnd(0);
			}
		}
		CVec4f zero_v4f, sum_v4f;
        zero_v4f.SetXYZW(0,0,0,0);
        sum_v4f = zero_v4f;
        int shared_i = 0;
       	for(int pc_i = 0; pc_i < points.Size(); pc_i++)
        {
        	for(int fc_i = 0; fc_i < faces.Size(); fc_i++)
         	{
          		if(faces[fc_i]->p[0] == pc_i || faces[fc_i]->p[1] == pc_i || faces[fc_i]->p[2] == pc_i)
            	{
             		sum_v4f = sum_v4f + fnormals[faces[fc_i]->fn];
                    // sum.x /= 2; sum.y /= 2; sum.z /= 2;
                 	shared_i++;
                    for(int nc_i = 0; nc_i < faces[fc_i]->p.Size(); nc_i++)
                    {
						if(faces[fc_i]->p[nc_i] == pc_i) faces[fc_i]->n[nc_i] = pc_i;
					}
                   	// if(faces[fc_i].p[0] == pc_i) faces[fc_i].n[0] = pc_i;
                    // if(faces[fc_i].p[1] == pc_i) faces[fc_i].n[1] = pc_i;
                    // if(faces[fc_i].p[2] == pc_i) faces[fc_i].n[2] = pc_i;
                }

               
            }
           	normals[pc_i].SetXYZW(sum_v4f.X() /-shared_i, sum_v4f.Y()/-shared_i, sum_v4f.Z()/-shared_i, 0);
           	normals[pc_i].Normalise();
            sum_v4f = zero_v4f;
            shared_i = 0;

        }
        
	}
	has_normals= true;
}

// Generate a list of edges, and which poly meets which poly
void CMesh::GenerateEdges()
{
	// for each face, determine its edges, and what faces those edges link to.
    int check_i = 0;
	for(int fc_i = 0; fc_i < faces.Size(); fc_i++)
	{
		for(int pc_i = 0; pc_i < faces[fc_i]->p.Size(); pc_i++)
		{
			// get two linked vertices
			int v1_i = pc_i;
			int v2_i = pc_i+1;
			if(pc_i == faces[fc_i]->p.Size()-1) v2_i = 0;
			
			// look through every face of the mesh to find one that shares both
			// these vertices
			for(int fc2_i = 0; fc2_i < faces.Size(); fc2_i++)
			{
				// if the face does share both vertices, create an edge.
				// but don't check against the same edge
                if(fc2_i != fc_i)
                {
					bool hasV1_b, hasV2_b;
					hasV1_b = hasV2_b = false;
					for(int pc2_i = 0; pc2_i < faces[fc2_i]->p.Size(); pc2_i++)
 					{
 						if((faces[fc2_i]->p[pc2_i]) == faces[fc_i]->p[v1_i])
 									hasV1_b = true;
	 					if((faces[fc2_i]->p[pc2_i]) == faces[fc_i]->p[v2_i])
 									hasV2_b = true;
 					}
 					if(hasV1_b && hasV2_b)
 					{
	                     // probably need to go through this and remove the duplicates...
 						CEdge temp_edge;
 						temp_edge.f[0] = fc_i;
 						temp_edge.f[1] = fc2_i;
 						temp_edge.p[0] = faces[fc_i]->p[v1_i];
 						temp_edge.p[1] = faces[fc_i]->p[v2_i];
	 					edges.AddToEnd(temp_edge);
	      				check_i++;
 					}
      			}
			}
		}	
	}
	fprintf(stderr, "%d\n", check_i);

}


void CMesh::CalcFaceNormals(bool in_Force_b)
{
	// done by cross product.
    for(int fc_i = 0; fc_i < faces.Size(); fc_i++)
    {
        CVec4f v0, v2v0, v2v1 , tfn;
        v0   = points[faces[fc_i]->p[0]];
        v2v0 = v0 - points[faces[fc_i]->p[2]];
        v2v1 = points[faces[fc_i]->p[2]] - points[faces[fc_i]->p[1]];
        
        tfn = v2v0.Cross(v2v1);

        tfn.Normalise();                                            // normalise normal
        fnormals.AddToEnd(tfn);                                     // add normal to array
        faces[fc_i]->fn = fnormals.Size()-1;						// save the index
    }
    if(fnormals.Size() > 0)has_fnormals = true;
}

void CMesh::PrepareForDestruction()
{
	for(int fc_i = 0; fc_i < faces.Size(); fc_i++)
	{
		faces[fc_i]->n.Clear();
		faces[fc_i]->p.Clear();
		faces[fc_i]->uv.Clear();
		delete(faces[fc_i]);
	}
	points.Clear();
	faces.Clear();
	uvpoints.Clear();
	normals.Clear();
	edges.Clear();
	colours.Clear();
	fnormals.Clear();
}

void CMesh::GenerateDisplayList()
{
	// before calling the display list, the correct texture must be set or unset.
	// List does not affect rendering properties, all things must be set prior to
	// calling the list. This just does the geometry data.
	// don't do this if you don't want normals 

	if(!has_normals) CalcNormals(true);

	dispList = glGenLists(1);
	glNewList(dispList, GL_COMPILE);
		// cycle faces
		for(int fc_i = 0; fc_i < faces.Size(); fc_i++)
		{
			CFace *face;
			face = faces[fc_i];
			// draw face
			glBegin(GL_POLYGON);
			// cycle points of face
			for(int pc_i = 0; pc_i < face->p.Size(); pc_i++)
			{
				glNormal3f(normals[face->n[pc_i]].X(),
						   normals[face->n[pc_i]].Y(),
					       normals[face->n[pc_i]].Z());
				
				glTexCoord2f(uvpoints[face->uv[pc_i]].X(),
							 uvpoints[face->uv[pc_i]].Y());
				
				// anything else?
				// and draw it
				glVertex3f(points[face->p[pc_i]].X(),
						   points[face->p[pc_i]].Y(),
						   points[face->p[pc_i]].Z());
	            
			}
			glEnd();
		}
		glEndList();

		has_displayList = true;
}