/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DAZZLE_MESH
#define DAZZLE_MESH

#include "Matlib.h"
#include "Face.h"
#include "Edge.h"
#include "Collision.h"
#include "DynArray.h"
#include "GL/gl.h"
#include "GL/glu.h"
#include <string>
using namespace std;

struct SColour
{
	float r;
	float g;
	float b;
	float a;
};

// A 3d mesh is a collection of faces, defined by points in space
// as such, we will have a list of points, texture points, vertex normals
// face normals, and faces.
class CMesh
{
public:
	CMesh(void);
	~CMesh(void);
	
	CDynArray<CVec4f>	points;		// vertices
	CDynArray<CVec2f>	uvpoints;	// texture coordinates
	CDynArray<CVec4f>	normals;	// vertex normals
	CDynArray<CVec4f>	fnormals;	// face normals
	CDynArray<SColour>	colours;	// vertex colors, same index as vertex
	CDynArray<CFace*>	faces;		// faces
	CDynArray<CEdge>	edges;		// edges
	GLuint dispList;				// display list, if mesh has

	string filename;		// file loaded from

	bool has_points;
	bool has_uvpoints;
	bool has_normals;
	bool has_fnormals;
	bool has_colors;
	bool has_faces;
	bool has_edges;
	bool has_displayList;
	
	// calculate the vertex normals
	void CalcNormals(bool in_Force_b);
	// Generate the edges so that the shadow algorithm works.
	void GenerateEdges();

	// check for a collision with a sphere (point and radius) in local space
	bool CollideMeshSphere(CVec4f in_SphOrigin_v4f, float in_SphRadius_f, SMeshCollision *out_Collision);

	// create a display list to speed up rendering
	void GenerateDisplayList();

	// memory clean up
	void PrepareForDestruction();

protected:
	// calculate the face normals, used by CalcNormals;
	void CalcFaceNormals(bool in_Force_b);
	
	// calculate for a given face the smallest/largest X,Y,Z values
	inline float SmallestX(int in_Face_i);
	inline float SmallestY(int in_Face_i);
	inline float SmallestZ(int in_Face_i);
	inline float BiggestX(int in_Face_i);
	inline float BiggestY(int in_Face_i);
	inline float BiggestZ(int in_Face_i);

	

};

#endif
