/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ModelEntity.h"
#include "ModelManager.h"
#include "TextureManager.h"
#include "GL/gl.h"
#include "GL/glu.h"
#include "Mesh.h"
#include "Face.h"

CModelEntity::CModelEntity(void)
{
	hasTexture_b = false;
	hasBoundSph_b = false;
	mesh_i = -1;
	mesh_p = NULL;
	tex_i  = -1;
}

CModelEntity::~CModelEntity(void)
{
}

void CModelEntity::Render(void)
{
	// Set opengl matrix to entity position
	glPushMatrix();
	
	glTranslatef(pos_v4f.X(), pos_v4f.Y(), pos_v4f.Z());
	glRotatef(rot_v3f.X(), 1, 0, 0);
    glRotatef(rot_v3f.Y(), 0, 1, 0);
    glRotatef(rot_v3f.Z(), 0, 0, 1);
	
	// texturing...
	if(hasTexture_b == true)
	{
		extern CTextureManager texMan;
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, texMan.textures[tex_i].id_ui);
	}
	// lighting
	// anything else??

	// render the entity
	extern CModelManager modelMan;
	CMesh *mesh;
	if(mesh_i >= 0) mesh = modelMan.GetMesh(mesh_i);
	else mesh = mesh_p;
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	// cycle faces
	for(int fc_i = 0; fc_i < mesh->faces.Size(); fc_i++)
	{
		CFace *face;
		face = mesh->faces[fc_i];
		// draw face
		glBegin(GL_POLYGON);
		// cycle points of face
		for(int pc_i = 0; pc_i < face->p.Size(); pc_i++)
		{
			if(mesh->has_normals)
			{
				glNormal3f(mesh->normals[face->n[pc_i]].X(),
						   mesh->normals[face->n[pc_i]].Y(),
						   mesh->normals[face->n[pc_i]].Z());
			}
			// tex-coords
			if(mesh->has_uvpoints)
			{	
				glTexCoord2f(mesh->uvpoints[face->uv[pc_i]].X(),
							 mesh->uvpoints[face->uv[pc_i]].Y());
			}
			if(mesh->has_colors)
			{
			}
			// anything else?
			// and draw it
			glVertex3f(mesh->points[face->p[pc_i]].X(),
					   mesh->points[face->p[pc_i]].Y(),
					   mesh->points[face->p[pc_i]].Z());
            
		}
		glEnd();
		
				
	}
	if(hasTexture_b == true)
	{
		glDisable(GL_TEXTURE_2D);
	}
	// draw subentities
	for(int sec_i = 0; sec_i < sub_ents.Size(); sec_i++)
	{
		sub_ents[sec_i]->Render();
	}
    // reset OpenGL matrix to value prior to rendering this entity
	glPopMatrix();
}

CModelEntity	CModelEntity::GenerateShadowVolume(CLightEntity *in_Light_p)
{
	// work out the silhouette
	// go through the mesh's list of edges and find those that mark the boundary
	// between front and back-facing polygons in the mesh.
	// can determine front and back facing by performing a dot product of a vector to
	// the light source, and the face normal
	// vector to light source will be from entity's origin. If this don't work right
	// then do it from one of the edge vertices
	CMesh *mesh_p;
	extern CModelManager modelMan;
	mesh_p = modelMan.GetMesh(mesh_i);

	// transform light to entity coordinates or it wont work right or will it??
	// i.e. DO IT HERE!!
/***********************************************/
	// translate light to entity space
	
	CVec4f lpos_v4f;
	lpos_v4f = in_Light_p->GetPos();

	CMatrix4f tfm;
	glMatrixMode(GL_MODELVIEW_MATRIX);
	glPushMatrix();
		glTranslatef(X(), Y(), Z());
		glRotatef(this->XRot(), 1,0,0);
		glRotatef(this->YRot(), 0,1,0);
		glRotatef(this->ZRot(), 0,0,1);
		glGetFloatv(GL_MODELVIEW_MATRIX, tfm.m_af);
	glPopMatrix();
	tfm.Transpose();
	tfm.Invert();
	

	
	CVec4f e2l_v4f;	// vector entity to light
	lpos_v4f = lpos_v4f.Transform(tfm);
	
	

    
	
	
	
/***********************************************/

    vector<int> silEdges_vi;
	
	e2l_v4f = lpos_v4f;
	e2l_v4f.Normalise();
	for(int edgc_i = 0; edgc_i < mesh_p->edges.Size(); edgc_i++)
	{
		// determine if each face is front or back facing
		float res1_f, res2_f;
        CVec4f fn1_v4f;	// face normal of first face
        CVec4f fn2_v4f; // face normal of second face
        int f1_i, f2_i;		// indices of the faces in question
        f1_i = mesh_p->edges[edgc_i].f[0];
        f2_i = mesh_p->edges[edgc_i].f[1];
        
        fn1_v4f = mesh_p->fnormals[mesh_p->faces[f1_i]->fn];
        fn2_v4f = mesh_p->fnormals[mesh_p->faces[f2_i]->fn];
        
		res1_f = fn1_v4f.Dot(e2l_v4f);
		res2_f = fn2_v4f.Dot(e2l_v4f);

		// only really want to remember the edges where the first face is
		// the front facing one (as there should be two entries in the list
		// defining the same edge, but in opposite directions). This should
		// allows construction of the shadow volume mesh with properly formed
		// vertex winding for each polygon.
		if((res1_f >= 0) && (res2_f < 0))
		{
			// one dot product result is +ve, the other -ve,
			// hence we have an edge between a back facing and front facing poly
			// so remember it
			silEdges_vi.push_back(edgc_i);
		}
	}
	// so now have the edges that define the silhouette.
	// extrude silhouette back to form shadow volume.
	CMesh *shadmesh_p;	// will be the mesh for the shadow volume
	shadmesh_p = new(CMesh);
	shadmesh_p->points.Clear();

	shadmesh_p->faces.Clear();
	shadmesh_p->has_normals = false;
	shadmesh_p->has_uvpoints= false;
	
	for(unsigned int edgc_i = 0; edgc_i < silEdges_vi.size(); edgc_i++)
	{
		CVec4f dir1, dir2;	// direction vectors, light to verts
		CVec4f p[4];		// points for mesh
		CFace *face;		// new face for mesh. //dont need normals etc...
		face = new(CFace);
		p[0] = mesh_p->points[mesh_p->edges[silEdges_vi[edgc_i]].p[0]];
		p[1] = mesh_p->points[mesh_p->edges[silEdges_vi[edgc_i]].p[1]];
		dir1 = p[0] - lpos_v4f;
		dir1.Normalise();
		dir2 = p[1] - lpos_v4f;
		dir2.Normalise();
		// here, 100 is the size of the shadow volume. This is the brute force method
		// of doing this...
		p[2]   = p[1] + (dir2*100);
		p[3]   = p[0] + (dir1*100);

		for(int pc_i = 0; pc_i < 4; pc_i ++)
		{
			// build up the mesh a bit more each iteration
			int newIndex_i = shadmesh_p->points.Size();
			shadmesh_p->points.AddToEnd(p[pc_i]);
			face->p.AddToEnd(newIndex_i);
		}
		shadmesh_p->faces.AddToEnd(face);
		// is that the mesh made???
		
		
	}

	// make the entity
	CModelEntity shadEnt;
	shadEnt.pos_v4f = pos_v4f;		// set to current entity position
	shadEnt.mesh_p  = shadmesh_p;	// point to the shadowmesh
	shadEnt.mesh_i  = -1;			// set this to -1 so that it uses mesh_p instead
									// of looking in the modelmanager
	shadEnt.rot_v3f = rot_v3f;		// set to current entity rotation
 	// anything else need to be set here??
	// and return the entity
	return shadEnt;
	
}

// Load a mesh for this entity
bool CModelEntity::LoadMesh(string in_Filename_s, int in_Filetype_i)
{
	extern CModelManager modelMan;
	mesh_i = modelMan.LoadMesh(in_Filename_s, in_Filetype_i);
	if(mesh_i == -1) return false;
	mesh_p = modelMan.meshes[mesh_i];
	// mesh_p->CalcNormals(true); //now down by modelmanager
	// mesh_p->GenerateEdges();
	return true;
}
// Load a texture for this entity
bool CModelEntity::LoadTexture(string in_Filename_s, int in_Filetype_i)
{
	extern CTextureManager texMan;
	tex_i = texMan.LoadTexture(in_Filename_s, in_Filetype_i);
	if(tex_i == -1) return false;
	hasTexture_b = true;
	return true;
}

void CModelEntity::CalculateBoundSph()
{
	float maxX = 0;
	float maxY = 0;
	float maxZ = 0;

	if(mesh_p == NULL)
	{
		extern CModelManager modelMan;
		mesh_p = modelMan.GetMesh(mesh_i);
	}
	for(int pc_i = 0; pc_i < mesh_p->points.Size(); pc_i++)
	{
		if(mesh_p->points[pc_i].X() > maxX) maxX = mesh_p->points[pc_i].X();
		if(mesh_p->points[pc_i].Y() > maxY) maxY = mesh_p->points[pc_i].Y();
		if(mesh_p->points[pc_i].Z() > maxZ) maxZ = mesh_p->points[pc_i].Z();
	}
	boundSphRadius_f = max(max(maxX, maxY), maxZ);
	hasBoundSph_b = true;
	return;
}

