/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DAZZLE_MODEL_ENTITY
#define DAZZLE_MODEL_ENTITY
/********************************************************************
 *							     Dazzle								*
 *							model entity class						*
 *																	*
 *							   Murray Evans							*
 ********************************************************************/
#include "Entity.h"
#include "LightEntity.h"
#include "Mesh.h"

class CModelEntity :
	public CEntity
{
public:
	CModelEntity(void);
	~CModelEntity(void);

	// the big change
	int mesh_i;			// index into the model_manager's list of meshes
	CMesh *mesh_p;		// pointer to the model's mesh. Added mostly for
						// the sake of shadow volumes which won't have a mesh in
						// the model manager's library
	bool hasTexture_b;	// does this entity have a texture loaded?
	int tex_i;			// index into the texture_manger's list of textures

	// Generate a shadow volume for the stencil shadow's algorithm
	CModelEntity	GenerateShadowVolume(CLightEntity *in_Light_p);
	// and the render function
	void Render(void);
	// Load a mesh for this entity
	bool LoadMesh(string in_Filename_s, int in_Filetype_i);
	// Load a texture for this entity
	bool LoadTexture(string in_Filename_s, int in_Filetype_i);

	// bounding sphere-bounding sphere collision check, this entity and in_Entity
	bool BSBSCollision(CModelEntity *in_Entity);
	// bounding sphere of this colliding with mesh of in_Entity
	bool BSMeshCollision(CModelEntity *in_Entity, SEntityCollision *out_Col);
	// bounding sphere of this colliding with vector in world space
	// pass in vector start point and vector end point
	bool BSVectCollision(CVec4f in_VectSP, CVec4f in_VectEP);


	// calculate the bounding sphere
    virtual void  CalculateBoundSph();
	float boundSphRadius_f;
	bool  hasBoundSph_b;
	bool  inContact_b;
	CModelEntity *contactThis;

	
};
#endif
