/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ModelManager.h"

CModelManager modelMan;

CModelManager::CModelManager(void)
{
}

CModelManager::~CModelManager(void)
{
	ClearLibrary();
}

// Load a mesh from the file in_Filename_s, of filetype defined by one of the above constants. Return index of loaded file.
int CModelManager::LoadMesh(string in_Filename_s, int in_Filetype_i)
{
	// make sure this mesh is not already loaded. If it is, return it's index
	for(int mc_i = 0; mc_i < this->meshes.Size(); mc_i++)
	{
		if(meshes[mc_i]->filename == in_Filename_s) return mc_i;
	}
	
	if(in_Filetype_i == OBJ)
	{
		if(LoadOBJ(in_Filename_s) < 0)
		{
			return -1;
		}
		else 
		{
			meshes[meshes.Size() - 1]->filename = in_Filename_s;
			meshes[meshes.Size() - 1]->GenerateEdges();
			meshes[meshes.Size() - 1]->CalcNormals(true);
			return meshes.Size() - 1;
		}
	}
	return 0;
}
// Add a mesh
int CModelManager::AddMesh(CMesh *in_Mesh)
{
	meshes.AddToEnd(in_Mesh);
	return meshes.Size()-1;
}

// Clear all loaded meshes
void CModelManager::ClearLibrary(void)
{
	for(int mc_i = 0; mc_i < meshes.Size(); mc_i++)
	{
		meshes[mc_i]->PrepareForDestruction();
		delete(meshes[mc_i]);
	}
	meshes.Clear();
}

// Get a pointer to the specified mesh
CMesh* CModelManager::GetMesh(int in_Index_i)
{
	if((in_Index_i >= 0) && (in_Index_i < meshes.Size()))
	{
		return meshes[in_Index_i];
	}
	else return NULL;
}
