/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DAZZLE_SCENE
#define DAZZLE_SCENE
/********************************************************************
 *								 Dazzle								*
 *							Basic Scene Class						*
 *																	*
 *							   Murray Evans							*
 ********************************************************************/

#include "DynArray.h"
#include "Entity.h"
#include "ModelEntity.h"
#include "LightEntity.h"
#include "Camera.h"
using namespace std;

class CScene
{
public:
	CScene(void);
	~CScene(void);

	// scene graph builds from this vector of entities, each relative to the world
	// an entity is a base class, lights, actors, avatars etc all inherit from
	// the base entity class.
	CDynArray<CEntity*> ents;
	// index into entity vector of entities that can be termed lights.
	CDynArray<int>	 lights;
	CDynArray<int>	 ments;		// index into vector of entities, that has a polygon mesh
	CLightEntity	 ambient;	// ambient light details
	CCamera camera;

	// Rendering options. Will grow as the renderer evolves.
	bool lightsEnabled_b;
	bool texturesEnabled_b;
	bool shadowsEnabled_b;

	// Add an entity to the scene. Return the index of the entity once it has been added
	int AddEntity(CEntity* in_Entity_p);
	int AddLight(CEntity* in_Entity_p);
	int AddMent(CEntity* in_Entity_p);

	// Load the scene from a scenefile, which details all entities that need be loaded, and anything special about them.
	virtual int LoadFromFile(string in_Filename_s);
	// Update the scene
	void Update(void);
	// Render the scene according to the render properties
	virtual int Render(void);
};

#endif