/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "TerrainGenerator.h"
#include "ModelManager.h"
#include "TextureManager.h"
#include "pixels.h"

CTerrainGenerator::CTerrainGenerator(void)
{
}

CTerrainGenerator::~CTerrainGenerator(void)
{
}

CModelEntity CTerrainGenerator::GenerateFromGrey(string in_HMapFilename_s, string in_TMapFilename_s,
												float in_MWidth_i, int in_MHeight_i, int in_MDepth_i,
												int in_DivisionsW_i, int in_DivisionsD_i)
{
	fprintf(stderr, "Generating terrain. Please be patient.\n");
	CModelEntity out_modent;
	// load image file
	SDL_Surface *hmap = NULL;
	hmap = SDL_LoadBMP(in_HMapFilename_s.c_str());
	// create empty mesh
	CMesh *mesh;
	mesh = new(CMesh);
	GenerateMesh(mesh, in_MWidth_i, in_MHeight_i, in_MDepth_i, in_DivisionsW_i, in_DivisionsD_i);
    	// set heights in mesh
	SetHeights(mesh, hmap, in_MHeight_i);
	// build entity with mesh. Does new entity have pointer to mesh, or add mesh to modelman?
	// mesh.GenerateEdges();
	mesh->CalcNormals(true);
	mesh->has_uvpoints = true;
	extern CModelManager modelMan;
	extern CTextureManager texMan;
	out_modent.mesh_i = modelMan.AddMesh(mesh);
	out_modent.SetPos(0,0,0);
	out_modent.SetRotation(0,0,0);
	out_modent.SetVelocity(0,0,0);
	out_modent.mesh_p = modelMan.GetMesh(out_modent.mesh_i);
	out_modent.tex_i  = texMan.LoadTexture(in_TMapFilename_s.c_str(), WIN_BMP);
	if(out_modent.tex_i > -1) out_modent.hasTexture_b = true;
	fprintf(stderr, "Terrain generation complete. Future project: Save terrain map as mesh?\n");
	// return entity
	return out_modent;
}

void CTerrainGenerator::GenerateMesh(CMesh *out_Mesh, int in_MWidth_i, int in_MHeight_i, int in_MDepth_i,
									  int in_DivisionsW_i, int in_DivisionsD_i)
{
	fprintf(stderr, "Generating Mesh.\n");
	// mesh has in_DivisionsW_i * int in_DivisionsD_i vertices
	// create vertices (three divisions needs four verts, etc...) hence +1
	float zinc_f, xinc_f;	// how far between each vertex?
	zinc_f = (float) in_MDepth_i / (float)(in_DivisionsD_i);
	xinc_f = (float) in_MWidth_i / (float)(in_DivisionsW_i);
	float txinc_f, tyinc_f;		// how far between texture vertices?
	txinc_f = 1.0/(float)in_DivisionsW_i;
	tyinc_f = 1.0/(float)in_DivisionsD_i;
	float tx_f = 0;
	float ty_f = 0;
	float zp_f = -(in_MDepth_i / 2);
	for(int xc_i = 0; xc_i < in_DivisionsW_i+1; xc_i++)
	{
		float xp_f = -(in_MWidth_i / 2);
		tx_f = 0;
		
		for(int zc_i = 0; zc_i < in_DivisionsD_i + 1; zc_i++)
		{
			// make the vertex and add it to the mesh
			CVec4f vert_v4f;
			vert_v4f.SetXYZW(xp_f, 0, zp_f, 1);
			out_Mesh->points.AddToEnd(vert_v4f);
			// calculate a texture coordinate and add it to the mesh
			CVec2f tvert_v2f;
			tvert_v2f.SetXY(tx_f, ty_f);
			out_Mesh->uvpoints.AddToEnd(tvert_v2f);
			// increment depth position
			xp_f += xinc_f;
			tx_f += txinc_f;
		}
		// increment z position
		zp_f += zinc_f;
		ty_f += tyinc_f;
	}
	// now, need to generate face information
	// 0     1     2
	// @-----@-----@
	// |    /|    /|
	// |   / |   / |			in_DivisionsD_i == 2
	// |  /  |  /  |			in_DivisionsW_i == 2
	// | /	 | /   |			gives 9 vertices, 8*2 faces
	// |/    |/    |			how to build this ??
	// @-----@-----@ 5
	// |    /|    /|
	// |   / |   / |
	// |  /  |  /  |
	// | /	 | /   |
	// |/    |/    |
	// @-----@-----@ 8

	// cycle all but the last row of vertices
	int numIter_i = (in_DivisionsW_i+1)*in_DivisionsD_i;
	int skipCount = 0;
	int skipValue = in_DivisionsD_i+1;
	for(int vc_i = 0; vc_i < numIter_i; vc_i++)
	{
		//skip first column of vertices
		//if((vc_i % (in_DivisionsD_i+1)) != 0)
		if((skipCount != skipValue) && (skipCount != 0))
		{
			// two faces per quad
			CFace *temp;
			temp = new(CFace);
			temp->p.AddToEnd(vc_i);
			temp->p.AddToEnd(vc_i-1);
			temp->p.AddToEnd(vc_i+in_DivisionsW_i);

			CFace *temp2;
			temp2 = new(CFace);
			temp2->p.AddToEnd(vc_i);
			temp2->p.AddToEnd(vc_i + in_DivisionsW_i);
			temp2->p.AddToEnd(vc_i + in_DivisionsW_i+1);

			temp->uv.AddToEnd(vc_i);
			temp->uv.AddToEnd(vc_i-1);
			temp->uv.AddToEnd(vc_i+in_DivisionsW_i);

			temp2->uv.AddToEnd(vc_i);
			temp2->uv.AddToEnd(vc_i + in_DivisionsW_i);
			temp2->uv.AddToEnd(vc_i + in_DivisionsW_i+1);
			out_Mesh->faces.AddToEnd(temp);
			out_Mesh->faces.AddToEnd(temp2);
		}
		else skipCount = 0;
        skipCount++;
	}
	out_Mesh->has_uvpoints = true;
	out_Mesh->has_normals  = true;
	
}

void CTerrainGenerator::SetHeights(CMesh *in_Mesh, SDL_Surface *in_Surf, int in_MaxHeight_f)
{
	// Each vertex has texture coord with same index
	// Get colour value of tex coord from surface. Use to set height value for vertex
	fprintf(stderr, "Setting heights.\n");
	float factor_f = in_MaxHeight_f/255;
	SDL_LockSurface(in_Surf);
	int numIter_i = in_Mesh->points.Size();
	float w,h;
	w = (float)in_Surf->w;
	h = (float)in_Surf->h;
	for(int vc_i = 0; vc_i < numIter_i; vc_i++)
	{
		Uint8 col_ui;
		float tx_f, ty_f;
		float height_f;

		tx_f = in_Mesh->uvpoints[vc_i].X() * w;
		ty_f = in_Mesh->uvpoints[vc_i].Y() * h;
//		fprintf(stderr, "X: %f \t Y: %f \n", in_Mesh->uvpoints[vc_i].X(), in_Mesh->uvpoints[vc_i].Y()); 
		col_ui = getpixel(in_Surf, tx_f, ty_f);
		height_f = factor_f * col_ui;
		in_Mesh->points[vc_i].SetY(height_f);

		//fprintf(stderr, "height: %f \t colour: %d\n", height_f, col_ui);
		//string str = in_Mesh->points[vc_i].WriteOut();
		//fprintf(stderr, "%s\n", str.c_str());
		
	}
	SDL_UnlockSurface(in_Surf);
}
