/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "TextureManager.h"
#include <iostream>
/********************************************************************
 *							     Dazzle								*
 *							Texture manager							*
 *																	*
 *							   Murray Evans							*
 ********************************************************************/
CTextureManager texMan;

CTextureManager::CTextureManager(void)
{
}

CTextureManager::~CTextureManager(void)
{
	ClearLibrary();
}

// Load a texture from a file in_Filename_s. Filetype is one of the above defined constants
int CTextureManager::LoadTexture(string in_Filename_s, int in_FileType_i)
{
	SDL_Surface *image;
	image = NULL;
	STexture new_tex;
	// make sure texture is not already loaded, if it is, return its index
	for(unsigned int tc_i = 0; tc_i < textures.size(); tc_i++)
	{
		if(textures[tc_i].filename_s == in_Filename_s)
			return tc_i;
	}
	image = LoadImage(in_Filename_s, in_FileType_i);	// get an image file of given format
	//image = SDL_LoadBMP(in_Filename_s.c_str());
	if(image == NULL)
	{
		cout << "Error loading file: " << in_Filename_s << "." << endl;
		return -1;
	}
    CreateTexture(&new_tex, image, in_FileType_i);	// turn texture image into OpenGL texture
	new_tex.filename_s = in_Filename_s;
	textures.push_back(new_tex);					// add texture to library
	SDL_FreeSurface(image);
	return textures.size()-1;						// return index of new texture.
	
}

// Delete all stored texture data
void CTextureManager::ClearLibrary(void)
{
	for(unsigned int tc_i = 0; tc_i < textures.size(); tc_i++)
	{
		glDeleteTextures(1, &textures[tc_i].id_ui);
	}
	textures.clear();
}

SDL_Surface *CTextureManager::LoadImage(string in_Filename_s, int in_Filetype_i)
{
	SDL_Surface *out_surf;
	if(in_Filetype_i == WIN_BMP)
	{
		out_surf = SDL_LoadBMP(in_Filename_s.c_str());		
	}
	return out_surf;
}

void CTextureManager::CreateTexture(STexture *out_tex, SDL_Surface *in_surf, int in_Filetype_i)
{
	GLenum fmt;
	if(in_Filetype_i == WIN_BMP) fmt = GL_BGR; /*_EXT*/

	// have file as SDL surface, generate texture
    STexture texture;
    glGenTextures(1, &(out_tex->id_ui));
    glBindTexture(GL_TEXTURE_2D, out_tex->id_ui);
    
    // create gl texture, mipmapping if necessary
	SDL_LockSurface(in_surf);
    gluBuild2DMipmaps(GL_TEXTURE_2D,
                      in_surf->format->BytesPerPixel,
                      in_surf->w,
                      in_surf->h,
                      fmt,
                      GL_UNSIGNED_BYTE,
                      in_surf->pixels);
	SDL_UnlockSurface(in_surf);
	return;    
}
