/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/************************************************************************
 *						2D vector class - Matlib						*
 *																		*
 *						    Murray Evans 2003							*
 ************************************************************************/

#include "Vec2f.h"


CVec2f::CVec2f(void)
{
}

CVec2f::CVec2f(float in_X, float in_Y)
{
	xy_af[0] = in_X;
	xy_af[1] = in_Y;
}

CVec2f::~CVec2f(void)
{
}

float CVec2f::X()
{
	return xy_af[0];
}

float CVec2f::Y()
{
	return xy_af[1];
}

void CVec2f::SetX(float in_X)
{
	xy_af[0] = in_X;
}

void CVec2f::SetY(float in_Y)
{
	xy_af[1] = in_Y;
}

void CVec2f::SetXY(float in_X, float in_Y)
{
	xy_af[0] = in_X;
	xy_af[1] = in_Y;
}

float CVec2f::Dot(CVec2f in_Vec)
{
	return ((xy_af[1] * in_Vec.xy_af[1]) + (xy_af[0] * in_Vec.xy_af[0]));
}

void CVec2f::Normalise()
{
	// if we have a 0 vector then it's hard to normalise, so just leave it alone
	if((xy_af[0] != 0) && (xy_af[1] != 0))
	{
		float mag_f;
		mag_f = Magnitude();
		mag_f = 1/Magnitude();
		xy_af[0] *= mag_f;
		xy_af[1] *= mag_f;
	}
}

float CVec2f::Magnitude()
{
	// magnitude is square root of dot product with self
	return sqrt(Dot(*this));
}

float CVec2f::MagSquare()
{
	return Dot(*this);
}

string CVec2f::WriteOut()
{
	string out_s;
	out_s = "|" + Cast<string> (xy_af[0]) + "|\n|" + Cast<string> (xy_af[1]) + "|\n";
	return out_s;
}

// Overload operator +
CVec2f CVec2f::operator+(int in_i)
{
	CVec2f out_v2f;
	out_v2f.xy_af[0] += in_i;
	out_v2f.xy_af[1] += in_i;
	return out_v2f;
}

CVec2f CVec2f::operator+(CVec2f in_v2f)
{
	CVec2f out_v2f;
	out_v2f.xy_af[0] += in_v2f.xy_af[0];
	out_v2f.xy_af[1] += in_v2f.xy_af[1];
	return out_v2f;
}

// Overload operator -
CVec2f CVec2f::operator-(int in_i)
{
	CVec2f out_v2f;
	out_v2f.xy_af[0] -= in_i;
	out_v2f.xy_af[1] -= in_i;
	return out_v2f;
}

CVec2f CVec2f::operator-(CVec2f in_v2f)
{
	CVec2f out_v2f;
	out_v2f.xy_af[0] -= in_v2f.xy_af[0];
	out_v2f.xy_af[1] -= in_v2f.xy_af[1];
	return out_v2f;
}

// Overload operator *
CVec2f CVec2f::operator*(int in_i)
{
	CVec2f out_v2f;
	out_v2f.xy_af[0] *= in_i;
	out_v2f.xy_af[1] *= in_i;
	return out_v2f;
}

// Overload operator /
CVec2f CVec2f::operator/(int in_i)
{
	CVec2f out_v2f;
	out_v2f.xy_af[0] /= in_i;
	out_v2f.xy_af[1] /= in_i;
	return out_v2f;
}

