/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/************************************************************************
 *						2D vector class - Matlib						*
 *																		*
 *						    Murray Evans 2003							*
 ************************************************************************/

#include "Vec3f.h"


CVec3f::CVec3f(void)
{
}

CVec3f::CVec3f(float in_X, float in_Y, float in_Z)
{
	xyz_af[0] = in_X;
	xyz_af[1] = in_Y;
	xyz_af[2] = in_Z;
}

CVec3f::~CVec3f(void)
{
}


float CVec3f::X()
{
	return xyz_af[0];
}

float CVec3f::Y()		
{
	return xyz_af[1];
}

float CVec3f::Z()
{
	return xyz_af[2];
}

void CVec3f::SetX(float in_X)
{
	xyz_af[0] = in_X;
}
void CVec3f::SetY(float in_Y)
{
	xyz_af[1] = in_Y;
}
void CVec3f::SetZ(float in_Z)
{
	xyz_af[2] = in_Z;
}
void CVec3f::SetXYZ(float in_X, float in_Y, float in_Z)
{
	xyz_af[0] = in_X;
	xyz_af[1] = in_Y;
	xyz_af[2] = in_Z;
}

float CVec3f::Dot(CVec3f in_Vec)
{
	return ((xyz_af[1] * in_Vec.xyz_af[1]) + (xyz_af[0] * in_Vec.xyz_af[0]) + (xyz_af[2] * in_Vec.xyz_af[2]));
}

CVec3f CVec3f::Cross(CVec3f in_Vec)
{
	CVec3f out_v3f;
	out_v3f.xyz_af[0] = (xyz_af[1]*in_Vec.xyz_af[2]) - (xyz_af[2]*in_Vec.xyz_af[1]);
	out_v3f.xyz_af[1] = (xyz_af[2]*in_Vec.xyz_af[0]) - (xyz_af[0]*in_Vec.xyz_af[2]);
	out_v3f.xyz_af[2] = (xyz_af[0]*in_Vec.xyz_af[1]) - (xyz_af[1]*in_Vec.xyz_af[0]);
	return out_v3f;
}

void CVec3f::Normalise()
{
	// if we have a 0 vector then it's hard to normalise, so just leave it alone
	if((xyz_af[0] != 0) && (xyz_af[1] != 0) && (xyz_af[3] != 0))
	{
		float mag_f;
		mag_f = Magnitude();
		mag_f = 1/Magnitude();
		xyz_af[0] *= mag_f;
		xyz_af[1] *= mag_f;
		xyz_af[2] *= mag_f;
	}
}

float CVec3f::Magnitude()
{
	// magnitude is square root of dot product with self
	return sqrt(Dot(*this));
}

float CVec3f::MagSquare()
{
	return Dot(*this);
}

string CVec3f::WriteOut()
{
	string out_s;
	out_s = "|" + Cast<string> (xyz_af[0]) + "|\n|" + Cast<string> (xyz_af[1]) + "|\n|" + Cast<string> (xyz_af[2]) + "|\n";
	return out_s;
}

// Operator overloading
// Overload operator +
CVec3f CVec3f::operator+(int in_i)
{
	CVec3f out_v3f;
	out_v3f.xyz_af[0] += in_i;
	out_v3f.xyz_af[1] += in_i;
	out_v3f.xyz_af[2] += in_i;
	return out_v3f;
}

CVec3f CVec3f::operator+(CVec3f in_v3f)
{
	CVec3f out_v3f;
	out_v3f.xyz_af[0] += in_v3f.xyz_af[0];
	out_v3f.xyz_af[1] += in_v3f.xyz_af[1];
	out_v3f.xyz_af[2] += in_v3f.xyz_af[2];
	return out_v3f;
}

// Overload operator -
CVec3f CVec3f::operator-(int in_i)
{
	CVec3f out_v3f;
	out_v3f.xyz_af[0] -= in_i;
	out_v3f.xyz_af[1] -= in_i;
	out_v3f.xyz_af[2] -= in_i;
	return out_v3f;
}

CVec3f CVec3f::operator-(CVec3f in_v3f)
{
	CVec3f out_v3f;
	out_v3f.xyz_af[0] -= in_v3f.xyz_af[0];
	out_v3f.xyz_af[1] -= in_v3f.xyz_af[1];
	out_v3f.xyz_af[2] -= in_v3f.xyz_af[2];
	return out_v3f;
}

// Overload operator *
CVec3f CVec3f::operator*(int in_i)
{
	CVec3f out_v3f;
	out_v3f.xyz_af[0] *= in_i;
	out_v3f.xyz_af[1] *= in_i;
	out_v3f.xyz_af[2] *= in_i;
	return out_v3f;

}

// Overload operator /
CVec3f CVec3f::operator/(int in_i)
{
	CVec3f out_v3f;
	out_v3f.xyz_af[0] /= in_i;
	out_v3f.xyz_af[1] /= in_i;
	out_v3f.xyz_af[2] /= in_i;
	return out_v3f;
}


