/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MATLIB_VEC3F
#define MATLIB_VEC3F

/************************************************************************
 *						3D vector class - Matlib						*
 *																		*
 *						    Murray Evans 2003							*
 ************************************************************************/
#include <cmath>
#include <string>
#include "Cast.h"	// a function that casts using a string stream
using namespace std;

class CVec3f
{
public:
	// constructors
	CVec3f(void);
	CVec3f(float in_X, float in_Y, float in_Z);
	~CVec3f(void);

	// data storage
	float xyz_af[3];

	// accessor
	float X();		// return x
	float Y();		// return y
	float Z();		// return z

	// setting
	void SetX(float in_X);
	void SetY(float in_Y);
	void SetZ(float in_Z);
	void SetXYZ(float in_X, float in_Y, float in_Z);

	// maths functions
	float  Dot(CVec3f in_Vec);			// dot product of this vector with in_Vec
	CVec3f Cross(CVec3f in_Vec);		// cross product of this vector with in_Vec
//	void   Transform(CMatrix3f in_Mat);	// transform by the square matrix in_Mat
	void   Normalise();					// Normalise the vector (set magnitude to 1)
	float  Magnitude();					// Return length/magnitude of vector
	float  MagSquare();					// Return magnitude squared (faster than magnitude)

	// output as string
	string WriteOut();

	// Overloaded operators
	CVec3f operator+(int in_i);
	CVec3f operator+(CVec3f in_v3f);
	CVec3f operator-(int in_i);
	CVec3f operator-(CVec3f in_v3f);
	CVec3f operator*(int in_i);
	CVec3f operator/(int in_i);
};
#endif
