/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/************************************************************************
 *						4D vector class - Matlib						*
 *																		*
 *						    Murray Evans 2003							*
 ************************************************************************/
#include "Vec4f.h"

CVec4f::CVec4f(void)
{
}

CVec4f::CVec4f(float in_X, float in_Y, float in_Z, float in_W)
{
	xyzw_af[0] = in_X;
	xyzw_af[1] = in_Y;
	xyzw_af[2] = in_Z;
	xyzw_af[3] = in_W;
}

CVec4f::~CVec4f(void)
{
}


float CVec4f::X()
{
	return xyzw_af[0];
}

float CVec4f::Y()		
{
	return xyzw_af[1];
}

float CVec4f::Z()
{
	return xyzw_af[2];
}
float CVec4f::W()
{
	return xyzw_af[3];
}

void CVec4f::SetX(float in_X)
{
	xyzw_af[0] = in_X;
}
void CVec4f::SetY(float in_Y)
{
	xyzw_af[1] = in_Y;
}
void CVec4f::SetZ(float in_Z)
{
	xyzw_af[2] = in_Z;
}
void CVec4f::SetW(float in_W)
{
	xyzw_af[3] = in_W;
}
void CVec4f::SetXYZW(float in_X, float in_Y, float in_Z, float in_W)
{
	xyzw_af[0] = in_X;
	xyzw_af[1] = in_Y;
	xyzw_af[2] = in_Z;
	xyzw_af[3] = in_W;
}

float CVec4f::Dot(CVec4f in_Vec)
{
	return ((xyzw_af[1] * in_Vec.xyzw_af[1]) + (xyzw_af[0] * in_Vec.xyzw_af[0]) + (xyzw_af[2] * in_Vec.xyzw_af[2]) + (xyzw_af[3] * in_Vec.xyzw_af[3]));
}

// Assuming that the vec4f is homogeneous, so the cross product is as if it were
// a vec3 (i.e the x,y,z are crossed, the w is ignored)
CVec4f CVec4f::Cross(CVec4f in_Vec)
{
	CVec4f out_v4f;
	out_v4f.xyzw_af[0] = (xyzw_af[1]*in_Vec.xyzw_af[2]) - (xyzw_af[2]*in_Vec.xyzw_af[1]);
	out_v4f.xyzw_af[1] = (xyzw_af[2]*in_Vec.xyzw_af[0]) - (xyzw_af[0]*in_Vec.xyzw_af[2]);
	out_v4f.xyzw_af[2] = (xyzw_af[0]*in_Vec.xyzw_af[1]) - (xyzw_af[1]*in_Vec.xyzw_af[0]);
	out_v4f.xyzw_af[3] = 0;
	return out_v4f;
}
CVec4f CVec4f::Cross(CVec3f in_Vec)
{
	CVec4f out_v4f;
	out_v4f.xyzw_af[0] = (xyzw_af[1]*in_Vec.xyz_af[2]) - (xyzw_af[2]*in_Vec.xyz_af[1]);
	out_v4f.xyzw_af[1] = (xyzw_af[2]*in_Vec.xyz_af[0]) - (xyzw_af[0]*in_Vec.xyz_af[2]);
	out_v4f.xyzw_af[2] = (xyzw_af[0]*in_Vec.xyz_af[1]) - (xyzw_af[1]*in_Vec.xyz_af[0]);
	out_v4f.xyzw_af[3] = 0;
	return out_v4f;
}
void CVec4f::Normalise()
{
	// if we have a 0 vector then it's hard to normalise, so just leave it alone
	if(!((xyzw_af[0] == 0) && (xyzw_af[1] == 0) && (xyzw_af[3] == 0)))
	{
		float mag_f;
		float invmag_f;
		mag_f = Magnitude();
		invmag_f = 1/mag_f;
		xyzw_af[0] *= invmag_f;
		xyzw_af[1] *= invmag_f;
		xyzw_af[2] *= invmag_f;
		xyzw_af[3] *= invmag_f;
	}
}

float CVec4f::Magnitude()
{
	// magnitude is square root of dot product with self
	return sqrt(Dot(*this));
}

float CVec4f::MagSquare()
{
	return Dot(*this);
}

CVec4f CVec4f::Transform(CMatrix4f in_Mat)
{
	CVec4f out_v4f;
	out_v4f.SetX((in_Mat.m_af[0] * X()) + (in_Mat.m_af[1] * Y()) + (in_Mat.m_af[2] * Z()) + (in_Mat.m_af[3]));
	out_v4f.SetY((in_Mat.m_af[4] * X()) + (in_Mat.m_af[5] * Y()) + (in_Mat.m_af[6] * Z()) + (in_Mat.m_af[7]));
	out_v4f.SetZ((in_Mat.m_af[8] * X()) + (in_Mat.m_af[9] * Y()) + (in_Mat.m_af[10] * Z()) + (in_Mat.m_af[11]));
	out_v4f.SetW(1);
	return out_v4f;
}

string CVec4f::WriteOut()
{
	string out_s;
	out_s = "|" + Cast<string> (xyzw_af[0]) + "|\n|" + Cast<string> (xyzw_af[1]) + "|\n|" + Cast<string> (xyzw_af[2]) + "|\n|" + Cast<string> (xyzw_af[3])+ "|\n";
	return out_s;
}

// Operator overloading
// Overload operator +
CVec4f CVec4f::operator+(int in_i)
{
	CVec4f out_v4f = *this;
	out_v4f.xyzw_af[0] += in_i;
	out_v4f.xyzw_af[1] += in_i;
	out_v4f.xyzw_af[2] += in_i;
	out_v4f.xyzw_af[3] += in_i;
	return out_v4f;
}

CVec4f CVec4f::operator+(CVec4f in_v4f)
{
	CVec4f out_v4f = *this;
	out_v4f.xyzw_af[0] += in_v4f.xyzw_af[0];
	out_v4f.xyzw_af[1] += in_v4f.xyzw_af[1];
	out_v4f.xyzw_af[2] += in_v4f.xyzw_af[2];
	out_v4f.xyzw_af[3] += in_v4f.xyzw_af[3];
	return out_v4f;
}

// Overload operator -
CVec4f CVec4f::operator-(int in_i)
{
	CVec4f out_v4f = *this;
	out_v4f.xyzw_af[0] -= in_i;
	out_v4f.xyzw_af[1] -= in_i;
	out_v4f.xyzw_af[2] -= in_i;
	out_v4f.xyzw_af[3] -= in_i;
	return out_v4f;
}

CVec4f CVec4f::operator-(CVec4f in_v4f)
{
	CVec4f out_v4f = *this;
	out_v4f.xyzw_af[0] -= in_v4f.xyzw_af[0];
	out_v4f.xyzw_af[1] -= in_v4f.xyzw_af[1];
	out_v4f.xyzw_af[2] -= in_v4f.xyzw_af[2];
	out_v4f.xyzw_af[3] -= in_v4f.xyzw_af[3];
	return out_v4f;
}

// Overload operator *
CVec4f CVec4f::operator*(int in_i)
{
	CVec4f out_v4f = *this;
	out_v4f.xyzw_af[0] *= in_i;
	out_v4f.xyzw_af[1] *= in_i;
	out_v4f.xyzw_af[2] *= in_i;
	out_v4f.xyzw_af[3] *= in_i;
	return out_v4f;
}
CVec4f CVec4f::operator*(float in_f)
{
	CVec4f out_v4f = *this;
	out_v4f.xyzw_af[0] *= in_f;
	out_v4f.xyzw_af[1] *= in_f;
	out_v4f.xyzw_af[2] *= in_f;
	out_v4f.xyzw_af[3] *= in_f;
	return out_v4f;
}

// Overload operator /
CVec4f CVec4f::operator/(int in_i)
{
	CVec4f out_v4f= *this;
	out_v4f.xyzw_af[0] /= in_i;
	out_v4f.xyzw_af[1] /= in_i;
	out_v4f.xyzw_af[2] /= in_i;
	out_v4f.xyzw_af[3] /= in_i;
	return out_v4f;
}
