#include "pixels.h"
#include <cstdlib>
#include <cmath>
using namespace std;

/*
 * Return the pixel value at (x, y)
 * NOTE: The surface must be locked before calling this!
 */
Uint32 getpixel(SDL_Surface *surface, int x, int y)
{
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to retrieve */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        return *p;

    case 2:
        return *(Uint16 *)p;


    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
            return p[0] << 16 | p[1] << 8 | p[2];
        else
            return p[0] | p[1] << 8 | p[2] << 16;

    case 4:
        return *(Uint32 *)p;

    default:
        return 0;       /* shouldn't happen, but avoids warnings */
    }
}


/*
 * Set the pixel at (x, y) to the given value
 * NOTE: The surface must be locked before calling this!
 */
void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel)
{
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to set */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        *p = pixel;
        break;

    case 2:
        *(Uint16 *)p = pixel;
        break;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
            p[0] = (pixel >> 16) & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = pixel & 0xff;
        } else {
            p[0] = pixel & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = (pixel >> 16) & 0xff;
        }
        break;

    case 4:
        *(Uint32 *)p = pixel;
        break;
    }
}

int sgn(int i)
{
    if(i>=0) return 1;
    return -1;
}

void drawline(SDL_Surface *surface, int a, int b, int c, int d, int col)
{
    long u,s,v,d1x,d1y,d2x,d2y,m,n;
    int i;
    
    SDL_LockSurface(surface);
    u = c-a;
    v = d-b;
    d1x = sgn(u);
    d1y = sgn(v);
    d2x = sgn(u);
    d2y = 0;
    m = abs(u);
    n = abs(v);
    
    if (m <= n)
    {
        d2x = 0;
        d2y = sgn(v);
        m = abs(v);
        n = abs(u);
    }
    
    s = (int)(m/2);
    
    for(i=0;i<m;i++)
    {
        if((b>0) && (b<256) && (a>0) && (a < 256))
        {
             putpixel(surface,b,a,col);
        }
        s += n;
        if(s >=m)
        {
            s -= m;
            a += d1x;
            b += d1y;
        }
        else
        {
            a += d2x;
            b += d2y;
        }
    }
    SDL_UnlockSurface(surface);
}

