/***************************************************************************
 *   Copyright (C) 2004 by Murray Evans                                    *
 *   m.evans@rdg.ac.uk                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "Scene.h"
#include "GL/gl.h"
#include "GL/glu.h"
#include "SDL/SDL.h"
#include "pixels.h"

// Render the scene according to the render properties
int CScene::Render(void)
{
	// set up drawing and render each entity
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	// gluPerspective(45, 640/480, 1, 100);
	glFrustum(-1, 1, -0.75, 0.75, 1, 1000);
    glMatrixMode(GL_MODELVIEW);

	// enable depth testing if it's not already enabled.
    glEnable(GL_DEPTH_TEST);
	// set up camera
    CVec4f u, f, p;
    u = camera.GetUp();

    f = camera.GetFace();
    p = camera.GetPos();
	gluLookAt(p.X(),p.Y(),p.Z(),
			  p.X() + f.X(), p.Y() + f.Y(),p.Z()+f.Z(),
			  u.X(),u.Y(),u.Z());

	if(shadowsEnabled_b)
	{
		// render with ambient lighting
		if(lightsEnabled_b)
		{
			glEnable(GL_LIGHTING);
			glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient.ambient);
		}
		for(int ec_i = 0; ec_i < ents.Size(); ec_i++)
		{
			ents[ec_i]->Render();
		}
		// shadow rendering passes
		for(int lc_i = 0; lc_i < lights.Size(); lc_i++)
		{
			CLightEntity *light_p;
			light_p = (CLightEntity*)ents[lights[lc_i]];
			if(light_p->castsShadows_b)
			{
				// perform all that needs to be done for the shadow pass for
				// this light. Namely constructing a shadow volume, rendering it
				// then performing the lighting pass. Not too hard surely (?)...

				// clear stencil buffer, configure stencil test to always pass
				// configure depth test to pass only when less than what's already in
				// buffer
				glClearStencil(0);
                glClear(GL_STENCIL_BUFFER_BIT);
                glEnable(GL_STENCIL_TEST);
                // will compare stencil values to 0, but for now stencil test
                // always passes.
                glStencilFunc(GL_ALWAYS, 0, ~0);

                // draw to depth buffer if new pixel closer
                glDepthFunc(GL_LESS);

                // Only want to draw into the stencil buffer, so remove writes
                // to the color and dept buffers
                glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
                glDepthMask(GL_FALSE);

                // Shadow volumes rendered using different stencil opperations if they
                // are front or back facing, so need to enable face culling
                glEnable(GL_CULL_FACE);

                // for each entity, generate a shadow volume and render it to the
                // stencil buffer
				for(int ec_i = 0; ec_i < ments.Size(); ec_i++)
				{
					CModelEntity shadvol;	// shadow volume is a mesh like any other
                    CModelEntity *ment_p;
                    ment_p = (CModelEntity*) ents[ments[ec_i]];
					shadvol = ment_p->GenerateShadowVolume(light_p);
					
					// render shadow volume
					glCullFace(GL_BACK);
					glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
					shadvol.Render();

					glCullFace(GL_FRONT);
					glStencilOp(GL_KEEP, GL_KEEP, GL_DECR);
					shadvol.Render();
				}
				// and now, do another rendering pass for this light.
                // enable the light
                GLenum lconst = GL_LIGHT0 + lc_i;
				glFrontFace(GL_CCW);				
				CVec4f lpos_v4f;
				lpos_v4f = light_p->GetPos();
                float lpos_af[3] = {lpos_v4f.X(), lpos_v4f.Y(), lpos_v4f.X()};
    			glLightfv(lconst, GL_POSITION, lpos_af);
                glLightfv(lconst, GL_DIFFUSE, light_p->diffuse);
				glLightfv(lconst, GL_SPECULAR, light_p->specular);
                glEnable(lconst);
				
				glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
				glDepthFunc(GL_EQUAL);
				glStencilFunc(GL_EQUAL, 0, ~0);
				glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
				glEnable(GL_BLEND);
				glBlendFunc(GL_ONE, GL_ONE);
				glCullFace(GL_BACK);
				for(int ec_i = 0; ec_i < ments.Size(); ec_i++)
				{
					CModelEntity *ment_p;
                    ment_p = (CModelEntity*) ents[ments[ec_i]];
                    ment_p->Render();
					glColor4f(0,0,0,0);
				}

				// and reset rendering states
				glDepthMask(GL_TRUE);
				glDepthFunc(GL_LEQUAL);
				glStencilFunc(GL_ALWAYS, 0, ~0);
				glDisable(lconst);
				glDisable(GL_STENCIL_TEST);
				glDisable(GL_BLEND);
			}
			
		}
		
	}
	else
	{
		// enable all lights
		if(lightsEnabled_b)
		{
			glEnable(GL_LIGHTING);
			glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient.ambient);
			// and the rest of them
			for(int lc_i = 0; lc_i < lights.Size(); lc_i++)
			{
				if(lc_i < GL_MAX_LIGHTS)
				{
					GLenum lconst = GL_LIGHT0 + lc_i;
					glEnable(lconst);
					CLightEntity *light_p;
					light_p = (CLightEntity*)ents[lights[lc_i]];

					GLfloat pos[4];
					pos[0] = light_p->X();
					pos[1] = light_p->Y();
					pos[2] = light_p->Z();
					pos[3] = 1.0;
					glLightfv(lconst, GL_DIFFUSE, light_p->diffuse);
					glLightfv(lconst, GL_SPECULAR, light_p->specular);
					glLightfv(lconst, GL_POSITION, pos);
				}
			}
		}
		for(int ec_i = 0; ec_i < ents.Size(); ec_i++)
		{
			// a quick culling hack that'll work with the over head nature of 
			// freebooter, but needs more work to be a permenant feature of Dazzle
			/*CVec4f cam = camera.GetPos();
			float xdist, ydist;
			xdist = abs(ents[ec_i]->X() - cam.X());
			ydist = abs(ents[ec_i]->Y() - cam.Y());
			if((xdist < 100) && (ydist < 100))//*/
				ents[ec_i]->Render();
				glFlush();
		}
		
	}

	return 0;
}
